#! /bin/sh
#------------------------------------------------------------------------------
# /tmp/install.sh - installation file for webmail services on EIS/FAIR
#
# Copyright (c) 2001-2010 The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:    2002-12-01 jed
# Last Update: $Id: install.sh 24985 2010-07-13 18:46:39Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version..
#------------------------------------------------------------------------------

# read eislib etc.
. /var/install/include/eislib
. /var/install/include/check-eisfair-version
. /var/install/include/jedlib

module_name="webmail"
installfile=/var/run/webmail.install

mecho -info "running install.sh ..."

# update shared library file
update_jedlib ${module_name}

# set platform specific parameters
case ${EISFAIR_SYSTEM} in
    eisfair-1)
        # eisfair-1
        webmail_path=/var/webmail

        if [ ! -d ${webmail_path} ]
        then
            mkdir -p ${webmail_path}
        fi
        ;;
    *)
        # eisfair-2
        webmail_oldpath=/var/webmail
        webmail_path=/data/packages/webmail

        if [ ! -d ${webmail_path} ]
        then
            mkdir -p ${webmail_path}
        fi

        if [ -d ${webmail_oldpath} -a ! -d ${webmail_path} ]
        then
            # moving data directory
            mecho "moving data directory ${webmail_oldpath} to ${webmail_path} ..."
            mv ${webmail_oldpath} ${webmail_path}
        fi
        ;;
esac

# extract files to data directory
tar xzf /tmp/wm_bin_var.tgz -C ${webmail_path}
tar xzf /tmp/wm_common_var.tgz -C ${webmail_path}
rm -f /tmp/wm_bin_var.tgz
rm -f /tmp/wm_common_var.tgz

# menue entry does not exist - create it
mecho "adding ${module_name} module menu ..."

# add webmail entry
if [ -f /var/install/packages/mail ]
then
    # mail package found
    /var/install/bin/add-menu -menu "setup.services.mail.modules.menu" \
                                    "setup.services.mail.modules.${module_name}.menu" "Webmail administration"

    # update mail modules menu
    /var/install/config.d/mail.sh updatemodulesmenu

elif [ -f /var/install/packages/vmail ]
then
    # vmail package found
    /var/install/bin/add-menu -menu "setup.services.vmail.modules.menu" \
                                    "setup.services.mail.modules.${module_name}.menu" "Webmail administration"

    # update vmail modules menu
    /var/install/config.d/vmail.sh updatemodulesmenu
else
    # standalone installation, because no mail an vmail package found
    /var/install/bin/add-menu -menu "setup.services.menu" \
                                    "setup.services.mail.modules.${module_name}.menu" "Webmail Service"
fi
    
if [ -f ${installfile} ]
then
    # previous version found - upgrading
    mecho "loading previous configuration file ..."
    /var/install/config.d/${module_name}-update.sh update

    mecho
    mecho -warn "Please remember to reinstall optional locale packages"
    mecho -warn "because previously installed files have been deleted"
    mecho -warn "during the package update!"
    anykey
elif [ ! -f /etc/config.d/${module_name} ]
then
    # new installation - parameter check
    mecho "loading initial configuration file ..."

    # set platform specific parameters
    case ${EISFAIR_SYSTEM} in
        eisfair-1)
            # eisfair-1
            cp /etc/default.d/${module_name} ${installfile}
            ;;
        *)
            # eisfair-2
            # change default codepage from 'iso-8859-1' to 'utf-8'
            sed '/GENERAL_DEF_CHARSET=/s/iso-8859-1/utf-8/g' /etc/default.d/${module_name} > ${installfile}
            ;;
    esac

    /var/install/config.d/${module_name}-update.sh update
fi

if [ -f ${installfile} ]
then
    rm -f ${installfile}
fi

# configuration
mecho "creating ${module_name} configuration ..."
/var/install/bin/edit -apply "${module_name}" "Edit ${module_name} services configuration"

exit 0
