#!/bin/sh
#------------------------------------------------------------------------------
# /tmp/preinstall.sh - pre installation file for webmail services on EIS/FAIR
#
# Copyright (c) 2001-2011 The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:    2002-12-01 jed
# Last Update: $Id: preinstall.sh 28359 2011-05-15 13:47:38Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version..
#------------------------------------------------------------------------------

# file names and pathes
module_name="webmail"

installfile=/var/run/${module_name}.install
webmailfile=/etc/config.d/${module_name}
webmail_deinstallfile=/var/install/deinstall/${module_name}

colecho "running preinstall.sh ..." gn

# check base version
installed_version=`/var/install/bin/check-version base 1.7.3`

case $installed_version in
    not-installed|new ) # wrong version
        colecho "required base package v1.7.3 or newer not installed - installation aborted." w rd br
        /var/install/bin/anykey
        exit 1
        ;;
    * ) # installed / old
        ;;
esac

# read eislib
. /var/install/include/eislib

# check apache2 version
installed_version=`/var/install/bin/check-version apache2 1.3.4`

case $installed_version in
    not-installed|new ) # wrong version
        mecho -warn "required apache2 package v1.3.4 or newer not installed - installation aborted."
        anykey
        exit 1
        ;;
    * ) # installed / old
        ;;
esac


# check apache2_php5 version
installed_version=`/var/install/bin/check-version apache2_php5 1.3.1`

case $installed_version in
    not-installed|new ) # wrong version
        mecho -warn "required apache2_php5 package v1.3.1 or newer not installed - installation aborted."
        anykey
        exit 1
        ;;
    * ) # installed / old
        ;;
esac

# check mail version
no_mail=0
no_vmail=0

installed_version=`/var/install/bin/check-version mail 1.8.10`

case $installed_version in
    not-installed|new ) # wrong version
        no_mail=1
        ;;
    * ) # installed / old
        ;;
esac

if [ $no_mail -eq 1 ]
then
    # check vmail version
    installed_version=`/var/install/bin/check-version vmail 0.2.4`

    case $installed_version in
        not-installed|new ) # wrong version
            no_vmail=1
            ;;
        * ) # installed / old
            ;;
    esac
fi

if [ $no_mail -eq 1 -a $no_vmail -eq 1 ]
then
    mecho -warn "Neither a mail package v1.8.10 nor a vmail package v0.2.4 has been installed."
    mecho -warn "Keep in mind that you will only be able to use the webmail-Package if you"
    mecho -warn "configure it to connect to an external mail-server."
    mecho -warn "See parameters WEBMAIL_SERVER_IMAP_HOST and WEBMAIL_SERVER_SMTP_HOST."
    anykey
fi

# backup existing file
if [ -f $webmailfile ]
then
    # move old files to backup directory and create backup
    /var/install/bin/backup-file -quiet ${module_name}

    mecho "saving previous configuration file to $installfile ..."
    cp $webmailfile $installfile
fi

# deinstall old software
if [ -f $webmail_deinstallfile ]
then
    grep '^rm -r -f -v $testroot/var/webmail' $webmail_deinstallfile > /dev/null

    if [ $? -eq 0 ]
    then
        mecho "deinstall script will be patched ..."
        grep -v '^rm -r -f -v $testroot/var/webmail' $webmail_deinstallfile > $webmail_deinstallfile.tmp
        mv $webmail_deinstallfile.tmp $webmail_deinstallfile
        chmod 700 $webmail_deinstallfile
    fi

    mecho "deinstalling previously installed software ..."
    $webmail_deinstallfile update
fi    

exit 0
