#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/webmail-update.sh - update or generate new webmail configuration
#
# Copyright (c) 2001-2010 The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:    2003-03-08 jed
# Last Update: $Id: webmail-update.sh 26502 2010-10-29 08:37:26Z jed $
#
#   webmail-update.sh [import] - import global addresses from csv file
#                                ($webmail_global_addresses)
#   webmail-update.sh [export] - export global addresses to csv file
#                                ($webmail_global_addresses)
#   webmail-update.sh [merge]  - merge global addresses ($webmail_globaddr)
#                                and csv file ($webmail_global_addresses)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# read eislib etc.
. /var/install/include/configlib
. /var/install/include/check-eisfair-version
. /var/install/include/eislib
. /var/install/include/jedlib

#exec 2>./webmailup-trace-$$.log
#set -x

#----------------------------------------------------------------------------------------
# check if mail has been enabled
#----------------------------------------------------------------------------------------
check_installed_mail ()
{
    retval=1

    if [ -f ${mailfile} ]
    then
        # mail installed
        . ${mailfile}

        if [ "${START_MAIL}" = "yes" ]
        then
            # mail activated
            if [ "${1}" != "-quiet" ]
            then
                mecho "mail has been enabled ..."
            fi
            retval=0
        else
            # mail deactivated
            if [ "${1}" != "-quiet" ]
            then
                mecho -warn "mail has been disabled ..."
            fi
        fi
    fi

    return ${retval}
}

#----------------------------------------------------------------------------------------
# check if vmail has been enabled
#----------------------------------------------------------------------------------------
check_installed_vmail ()
{
    retval=1

    if [ -f ${vmailfile} ]
    then
        # mail installed
        . ${vmailfile}

        if [ "${START_VMAIL}" = "yes" ]
        then
            # vmail activated
            if [ "${1}" != "-quiet" ]
            then
                mecho "vmail has been enabled ..."
            fi
            retval=0
        else
            # vmail deactivated
            if [ "${1}" != "-quiet" ]
            then
                mecho -warn "vmail has been disabled ..."
            fi
        fi
    fi

    return ${retval}
}

#------------------------------------------------------------------------------
# rename variables
#------------------------------------------------------------------------------
rename_variables ()
{
    renamed=0
    mecho "renaming parameter(s) ..."

    # v1.2.11 - first multi-instances version
    if [ $WEBMAIL_N -le 1 ]
    then
        imax=1
    else
        imax=$WEBMAIL_N
    fi

    idx=1
    while [ $idx -le $imax ]
    do
        if [ ! -z "`grep ^WEBMAIL_SERVER_DOMAIN $source_conf`" ]
        then
            eval tmp_val='$WEBMAIL_SERVER_DOMAIN'

            mecho "- WEBMAIL_SERVER_DOMAIN -> WEBMAIL_${idx}_SERVER_DOMAIN"
            eval "WEBMAIL_${idx}_SERVER_DOMAIN=\"${tmp_val}\""
            renamed=1
        fi

        for VARNAME in NAME TITLE LOGO LOGO_WIDTH LOGO_HEIGHT SIGNOUT_PAGE DEF_LANGUAGE TOP_FRAME PROVIDER_NAME PROVIDER_URL
        do
            if [ ! -z "`grep ^WEBMAIL_ORGA_${VARNAME} $source_conf`" ]
            then
                eval tmp_val='$WEBMAIL_ORGA_'${VARNAME}

                mecho "- WEBMAIL_ORGA_${VARNAME} -> WEBMAIL_${idx}_ORGA_${VARNAME}"
                eval "WEBMAIL_${idx}_ORGA_${VARNAME}=\"${tmp_val}\""
                renamed=1
            fi
        done

        for VARNAME in MOVE_MSGS_TO_TRASH MOVE_MSGS_TO_SEND MOVE_MSGS_TO_DRAFT SPECIAL_LIST_FIRST SPECIAL_USE_COLOR AUTO_EXPUNGE UNSEEN_NOTIFY UNSEEN_TYPE
        do
            if [ ! -z "`grep ^WEBMAIL_FOLDER_${VARNAME} $source_conf`" ]
            then
                eval tmp_val='$WEBMAIL_FOLDER_'${VARNAME}

                mecho "- WEBMAIL_FOLDER_${VARNAME} -> WEBMAIL_${idx}_FOLDER_${VARNAME}"
                eval "WEBMAIL_${idx}_FOLDER_${VARNAME}=\"${tmp_val}\""
                renamed=1
            fi
        done

        for VARNAME in DEF_CHARSET LEFT_FRAME_WIDTH ALLOW_PRIORITY_USE ALLOW_RECEIPTS_USE ALLOW_IDENTITY_EDIT
        do
            if [ ! -z "`grep ^WEBMAIL_GENERAL_${VARNAME} $source_conf`" ]
            then
                eval tmp_val='$WEBMAIL_GENERAL_'${VARNAME}

                mecho "- WEBMAIL_GENERAL_${VARNAME} -> WEBMAIL_${idx}_GENERAL_${VARNAME}"
                eval "WEBMAIL_${idx}_GENERAL_${VARNAME}=\"${tmp_val}\""
                renamed=1
            fi
        done

        for VARNAME in USE_ALL DEFAULT N
        do
            if [ ! -z "`grep ^WEBMAIL_THEMES_${VARNAME} $source_conf`" ]
            then
                eval tmp_val='$WEBMAIL_THEMES_'${VARNAME}

                mecho "- WEBMAIL_THEMES_${VARNAME} -> WEBMAIL_${idx}_THEMES_${VARNAME}"
                eval "WEBMAIL_${idx}_THEMES_${VARNAME}=\"${tmp_val}\""
                renamed=1
            fi
        done

        eval themes_n='$WEBMAIL_'${idx}'_THEMES_N'

        if [ $themes_n -eq 0 ]
        then
            jmax=4
        else
            jmax=$themes_n
        fi

        jdx=1
        while [ $jdx -le $jmax ]
        do
            if [ ! -z "`grep ^WEBMAIL_THEMES_${jdx}_FILENAME $source_conf`" ]
            then
                eval fname_old='$WEBMAIL_THEMES_'${jdx}'_FILENAME'
                eval fname_new="WEBMAIL_${idx}_THEMES_${jdx}_FILENAME"

                mecho "- WEBMAIL_THEMES_${jdx}_FILENAME -> WEBMAIL_${idx}_THEMES_${jdx}_FILENAME"
                eval "$fname_new=\"$fname_old\""
                renamed=1
            fi

            jdx=`expr $jdx + 1`
        done

        for VARNAME in USE_ALL N ADMINUSER
        do
            if [ ! -z "`grep ^WEBMAIL_PLUGINS_${VARNAME} $source_conf`" ]
            then
                eval tmp_val='$WEBMAIL_PLUGINS_'${VARNAME}

                mecho "- WEBMAIL_PLUGINS_${VARNAME} -> WEBMAIL_${idx}_PLUGINS_${VARNAME}"
                eval "WEBMAIL_${idx}_PLUGINS_${VARNAME}=\"${tmp_val}\""
                renamed=1
            fi
        done

        eval plugins_n='$WEBMAIL_'${idx}'_PLUGINS_N'

        if [ $plugins_n -eq 0 ]
        then
            jmax=4
        else
            jmax=$plugins_n
        fi

        jdx=1
        while [ $jdx -le $jmax ]
        do
            if [ ! -z "`grep ^WEBMAIL_PLUGINS_${jdx}_DIRNAME $source_conf`" ]
            then
                eval dname_old='$WEBMAIL_PLUGINS_'${jdx}'_DIRNAME'
                eval dname_new="WEBMAIL_${idx}_PLUGINS_${jdx}_DIRNAME"

                mecho "- WEBMAIL_PLUGINS_${jdx}_DIRNAME -> WEBMAIL_${idx}_PLUGINS_${jdx}_DIRNAME"
                eval "$dname_new=\"$dname_old\""
                renamed=1
            fi

            jdx=`expr $jdx + 1`
        done

        for VARNAME in WRITEABLE N
        do
            if [ ! -z "`grep ^WEBMAIL_GLOBADDR_${VARNAME} $source_conf`" ]
            then
                eval tmp_val='$WEBMAIL_GLOBADDR_'${VARNAME}

                mecho "- WEBMAIL_GLOBADDR_${VARNAME} -> WEBMAIL_${idx}_GLOBADDR_${VARNAME}"
                eval "WEBMAIL_${idx}_GLOBADDR_${VARNAME}=\"${tmp_val}\""
                renamed=1
            fi
        done

        if [ ! -z "`grep ^WEBMAIL_GLOBADDR_N ${source_conf}`" ]
        then
            # parameter WEBMAIL_GLOBADDR_N found
            eval globaddr_n='$WEBMAIL_GLOBADDR_N'

            if [ $globaddr_n -eq 0 ]
            then
                jmax=1
            else
                jmax=$globaddr_n
            fi

            jdx=1
            while [ $jdx -le $jmax ]
            do
                if [ ! -z "`grep ^WEBMAIL_GLOBADDR_${jdx}_NICKNAME $source_conf`" ]
                then
                    eval nick_old='$WEBMAIL_GLOBADDR_'${jdx}'_NICKNAME'
                    eval nick_new="WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_NICKNAME"

                    mecho "- WEBMAIL_GLOBADDR_${jdx}_NICKNAME -> WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_NICKNAME"
                    eval "$nick_new=\"$nick_old\""
                    renamed=1
                fi

                if [ ! -z "`grep ^WEBMAIL_GLOBADDR_${jdx}_FIRSTNAME $source_conf`" ]
                then
                    eval first_old='$WEBMAIL_GLOBADDR_'${jdx}'_FIRSTNAME'
                    eval first_new="WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_FIRSTNAME"

                    mecho "- WEBMAIL_GLOBADDR_${jdx}_FIRSTNAME -> WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_FIRSTNAME"
                    eval "$first_new=\"$first_old\""
                    renamed=1
                fi

                if [ ! -z "`grep ^WEBMAIL_GLOBADDR_${jdx}_LASTNAME $source_conf`" ]
                then
                    eval last_old='$WEBMAIL_GLOBADDR_'${jdx}'_LASTNAME'
                    eval last_new="WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_LASTNAME"

                    mecho "- WEBMAIL_GLOBADDR_${jdx}_LASTNAME -> WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_LASTNAME"
                    eval "$last_new=\"$last_old\""
                    renamed=1
                fi

                if [ ! -z "`grep ^WEBMAIL_GLOBADDR_${jdx}_EMAIL $source_conf`" ]
                then
                    eval email_old='$WEBMAIL_GLOBADDR_'${jdx}'_EMAIL'
                    eval email_new="WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_EMAIL"

                    mecho "- WEBMAIL_GLOBADDR_${jdx}_EMAIL -> WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_EMAIL"
                    eval "$email_new=\"$email_old\""
                    renamed=1
                fi

                if [ ! -z "`grep ^WEBMAIL_GLOBADDR_${jdx}_INFO $source_conf`" ]
                then
                    eval info_old='$WEBMAIL_GLOBADDR_'${jdx}'_INFO'
                    eval info_new="WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_INFO"

                    mecho "- WEBMAIL_GLOBADDR_${jdx}_INFO -> WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_INFO"
                    eval "$info_new=\"$info_old\""
                    renamed=1
                fi

                jdx=`expr $jdx + 1`
            done
        fi

        # v1.4.10
        if [ ! -z "`grep ^WEBMAIL_${idx}_GLOBADDR_WRITEABLE ${source_conf}`" ]
        then
            eval write_old='$WEBMAIL_'${idx}'_GLOBADDR_WRITEABLE'
            eval write_new="WEBMAIL_${idx}_GLOBADDR_FILE_WRITEABLE"

            mecho "- WEBMAIL_${idx}_GLOBADDR_WRITEABLE -> WEBMAIL_${idx}_GLOBADDR_FILE_WRITEABLE"
            eval "${write_new}=\"${write_old}\""
            renamed=1
        fi

        if [ ! -z "`grep ^WEBMAIL_${idx}_GLOBADDR_N ${source_conf}`" ]
        then
            eval num_old='$WEBMAIL_'${idx}'_GLOBADDR_N'
            eval num_new="WEBMAIL_${idx}_GLOBADDR_FILE_N"

            mecho "- WEBMAIL_${idx}_GLOBADDR_N -> WEBMAIL_${idx}_GLOBADDR_FILE_N"
            eval "${num_new}=\"${num_old}\""
            renamed=1
        fi

        eval globaddr_n='$WEBMAIL_'${idx}'_GLOBADDR_FILE_N'

        if [ "${globaddr_n}" = "" ]
        then
            globaddr_n=0
        fi

        if [ ${globaddr_n} -eq 0 ]
        then
            jmax=1
        else
            jmax=${globaddr_n}
        fi

        jdx=1
        while [ ${jdx} -le ${jmax} ]
        do
            if [ ! -z "`grep ^WEBMAIL_${idx}_GLOBADDR_${jdx}_NICKNAME ${source_conf}`" ]
            then
                eval nick_old='$WEBMAIL_'${idx}'_GLOBADDR_'${jdx}'_NICKNAME'
                eval nick_new="WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_NICKNAME"

                mecho "- WEBMAIL_${idx}_GLOBADDR_${jdx}_NICKNAME -> WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_NICKNAME"
                eval "${nick_new}=\"${nick_old}\""
                renamed=1
            fi

            if [ ! -z "`grep ^WEBMAIL_${idx}_GLOBADDR_${jdx}_FIRSTNAME ${source_conf}`" ]
            then
                eval first_old='$WEBMAIL_'${idx}'_GLOBADDR_'${jdx}'_FIRSTNAME'
                eval first_new="WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_FIRSTNAME"

                mecho "- WEBMAIL_${idx}_GLOBADDR_${jdx}_FIRSTNAME -> WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_FIRSTNAME"
                eval "${first_new}=\"${first_old}\""
                renamed=1
            fi

            if [ ! -z "`grep ^WEBMAIL_${idx}_GLOBADDR_${jdx}_LASTNAME ${source_conf}`" ]
            then
                eval last_old='$WEBMAIL_'${idx}'_GLOBADDR_'${jdx}'_LASTNAME'
                eval last_new="WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_LASTNAME"

                mecho "- WEBMAIL_${idx}_GLOBADDR_${jdx}_LASTNAME -> WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_LASTNAME"
                eval "${last_new}=\"${last_old}\""
                renamed=1
            fi

            if [ ! -z "`grep ^WEBMAIL_${idx}_GLOBADDR_${jdx}_EMAIL ${source_conf}`" ]
            then
                eval email_old='$WEBMAIL_'${idx}'_GLOBADDR_'${jdx}'_EMAIL'
                eval email_new="WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_EMAIL"

                mecho "- WEBMAIL_${idx}_GLOBADDR_${jdx}_EMAIL -> WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_EMAIL"
                eval "${email_new}=\"${email_old}\""
                renamed=1
            fi

            if [ ! -z "`grep ^WEBMAIL_${idx}_GLOBADDR_${jdx}_INFO ${source_conf}`" ]
            then
                eval info_old='$WEBMAIL_'${idx}'_GLOBADDR_'${jdx}'_INFO'
                eval info_new="WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_INFO"

                mecho "- WEBMAIL_${idx}_GLOBADDR_${jdx}_INFO -> WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_INFO"
                eval "${info_new}=\"${info_old}\""
                renamed=1
            fi

            jdx=`expr ${jdx} + 1`
        done

        idx=`expr ${idx} + 1`
    done

    if [ ${renamed} -eq 1 ]
    then
        mecho -info "... read documentation for renamed parameter(s)!"
        anykey
    fi
}

#------------------------------------------------------------------------------
# modify variables
#------------------------------------------------------------------------------
modify_variables ()
{
    modified=0
    mecho "modifying parameter(s) ..."

    echo "$WEBMAIL_ORGA_LOGO"|grep -q "^../"

    if [ $? -eq 0 ]
    then
        WEBMAIL_ORGA_LOGO=`echo "$WEBMAIL_ORGA_LOGO"|sed 's/^\.\.\///'`
        mecho "- WEBMAIL_ORGA_LOGO, '../' has been removed because only absolute pathes from webmail root are now allowed!"
        modified=1
    fi

    if [ ${WEBMAIL_N} -le 1 ]
    then
        imax=1
    else
        imax=${WEBMAIL_N}
    fi

    idx=1
    while [ ${idx} -le ${imax} ]
    do
        if [ ! -z "`grep ^WEBMAIL_${idx}_SERVER_IMAP_AUTH ${source_conf}`" ]
        then
            eval auth_old='$WEBMAIL_'${idx}'_SERVER_IMAP_AUTH'

            case ${MAIL_INSTALLED} in
                mail)
                    # mail
                    ;;
                vmail)
                    # vmail
                    if [ "${auth_old}" != 'login' ]
                    then
                        eval "WEBMAIL_${idx}_SERVER_IMAP_AUTH='login'"
                        mecho "- WEBMAIL_${idx}_SERVER_IMAP_AUTH, '${auth_old}' -> 'login'"
                        modified=1
                    fi
                    ;;
                *)
                    # non-local
                    ;;
            esac
        fi

        if [ ! -z "`grep ^WEBMAIL_${idx}_SERVER_IMAP_TYPE ${source_conf}`" ]
        then
            eval type_old='$WEBMAIL_'${idx}'_SERVER_IMAP_TYPE'

            case ${MAIL_INSTALLED} in
                mail)
                    # mail
                    if [ "${type_old}" != 'uw' ]
                    then
                        eval "WEBMAIL_${idx}_SERVER_IMAP_TYPE='uw'"
                        mecho "- WEBMAIL_${idx}_SERVER_IMAP_TYPE, '${type_old}' -> 'uw'"
                        modified=1
                    fi
                    ;;
                vmail)
                    # vmail
                    # set platform specific parameters
                    case ${EISFAIR_SYSTEM} in
                        eisfair-1)
                            # eisfair-1
                            if [ "${type_old}" != 'courier' ]
                            then
                                eval "WEBMAIL_${idx}_SERVER_IMAP_TYPE='courier'"
                                mecho "- WEBMAIL_${idx}_SERVER_IMAP_TYPE, '${type_old}' -> 'courier'"
                                modified=1
                            fi
                            ;;
                        *)
                            # default to eisfair-2/eisxen-1
                            if [ "${type_old}" != 'dovecot' ]
                            then
                                eval "WEBMAIL_${idx}_SERVER_IMAP_TYPE='dovecot'"
                                mecho "- WEBMAIL_${idx}_SERVER_IMAP_TYPE, '${type_old}' -> 'dovecot'"
                                modified=1
                            fi
                            ;;
                    esac

                    ;;
                *)
                    # non-local
                    ;;
            esac
        fi

        if [ ! -z "`grep ^WEBMAIL_${idx}_SERVER_SMTP_AUTH ${source_conf}`" ]
        then
            eval auth_old='$WEBMAIL_'${idx}'_SERVER_SMTP_AUTH'

            case ${MAIL_INSTALLED} in
                mail)
                    # mail
                    ;;
                vmail)
                    # vmail
                    if [ "${auth_old}" != 'login' ]
                    then
                        eval "WEBMAIL_${idx}_SERVER_SMTP_AUTH='login'"
                        mecho "- WEBMAIL_${idx}_SERVER_IMAP_TYPE, '${auth_old}' -> 'login'"
                        modified=1
                    fi
                    ;;
                *)
                    # non-local
                    ;;
            esac
        fi

        # v1.4.18
        eval plugins_n='$WEBMAIL_'${idx}'_PLUGINS_N'

        if [ ${plugins_n} -eq 0 ]
        then
            jmax=1
        else
            jmax=${plugins_n}
        fi

        jdx=1
        while [ ${jdx} -le ${jmax} ]
        do
            eval dname=\$WEBMAIL_${idx}_PLUGINS_${jdx}_DIRNAME

            if [ "${dname}" = "address_add" ]
            then
                eval "WEBMAIL_${idx}_PLUGINS_${jdx}_DIRNAME='add_address'"
                mecho "- WEBMAIL_${idx}_PLUGINS_${jdx}_DIRNAME, '${dname}' -> 'add_address'"
                modified=1
            fi

            jdx=`expr ${jdx} + 1`
        done

        idx=`expr ${idx} + 1`
    done

    if [ $modified -eq 1 ]
    then
        mecho -info "... read documentation for modified parameter(s)!"
        anykey
    fi
}

#------------------------------------------------------------------------------
# add variables
#------------------------------------------------------------------------------
pre_add_variables ()
{
    added=0
    mecho "pre-adding new parameter(s) ..."

    # v1.3.1 - first multi-instances version
    if [ -z "`grep ^WEBMAIL_N $source_conf`" ]
    then
        mecho "- WEBMAIL_N='1'"
        WEBMAIL_N='1'
        added=1
    fi

    if [ $added -eq 1 ]
    then
        mecho -info "... read documentation for new parameter(s)!"
        anykey
    fi
}

#------------------------------------------------------------------------------
# add variables
#------------------------------------------------------------------------------
add_variables ()
{
    added=0
    mecho "adding new parameter(s) ..."

    # 0.99.6
    if [ -z "`grep -E \"^WEBMAIL_.*ORGA_PROVIDER_NAME\" $source_conf`" ]
    then
        mecho "- WEBMAIL_ORGA_PROVIDER_NAME='eisfair'"
        WEBMAIL_ORGA_PROVIDER_NAME='eisfair'
        added=1
    fi

    if [ -z "`grep ^WEBMAIL_.*ORGA_PROVIDER_URL $source_conf`" ]
    then
        mecho "- WEBMAIL_ORGA_PROVIDER_URL='http://www.eisfair.org/'"
        WEBMAIL_ORGA_PROVIDER_URL='http://www.eisfair.org/'
        added=1
    fi

    # 1.0.1
    if [ -z "`grep ^WEBMAIL_.*THEMES_DEFAULT $source_conf`" ]
    then
        mecho "- WEBMAIL_THEMES_DEFAULT='0'"
        WEBMAIL_THEMES_DEFAULT='0'
        added=1
    fi

    if [ -z "`grep ^WEBMAIL_.*GLOBADDR_WRITEABLE ${source_conf}`" -a \
         -z "`grep ^WEBMAIL_.*_GLOBADDR_FILE_WRITEABLE ${source_conf}`" ]
    then
        mecho "- WEBMAIL_GLOBADDR_WRITEABLE='no'"
        WEBMAIL_GLOBADDR_WRITEABLE='no'
        added=1
    fi

    # v1.2.11 - first multi-instances version
    # moved to function 'pre_add_variables'
  # if [ -z "`grep ^WEBMAIL_N $source_conf`" ]
  # then
  #     mecho "- WEBMAIL_N='1'"
  #     WEBMAIL_N='1'
  #     added=1
  # fi

    if [ -z "`grep ^WEBMAIL_1_DOCUMENT_ROOT $source_conf`" ]
    then
        mecho "- WEBMAIL_1_DOCUMENT_ROOT='/var/www/htdocs/webmail'"
        WEBMAIL_1_DOCUMENT_ROOT='/var/www/htdocs/webmail'
        added=1
    fi

    if [ -z "`grep ^WEBMAIL_1_ACTIVE $source_conf`" ]
    then
        mecho "- WEBMAIL_1_ACTIVE='yes'"
        WEBMAIL_1_ACTIVE='yes'
        added=1
    fi

    # v1.2.11 - first multi-instances version
    if [ "${WEBMAIL_N}" = "" ]
    then
        WEBMAIL_N=1
    fi

    if [ ${WEBMAIL_N} -le 1 ]
    then
        imax=1
    else
        imax=${WEBMAIL_N}
    fi

    idx=1
    while [ ${idx} -le ${imax} ]
    do
        # v1.4.10
        eval globaddr_n='$WEBMAIL_'${idx}'_GLOBADDR_FILE_N'

        if [ ${globaddr_n} -eq 0 ]
        then
            jmax=1
        else
            jmax=$globaddr_n
        fi

        #----------------------------------------------------------------------

        if [ -z "`grep ^WEBMAIL_${idx}_GLOBADDR_LDAP_N ${source_conf}`" ]
        then
            mecho "- WEBMAIL_${idx}_GLOBADDR_LDAP_N='0'"
            eval "WEBMAIL_${idx}_GLOBADDR_LDAP_N='0'"
            added=1
        fi

        if [ -z "`grep ^WEBMAIL_${idx}_SERVER_IMAP_HOST ${source_conf}`" ]
        then
            mecho "- WEBMAIL_${idx}_SERVER_IMAP_HOST='localhost'"
            eval "WEBMAIL_${idx}_SERVER_IMAP_HOST='localhost'"
            added=1
        fi

        if [ -z "`grep ^WEBMAIL_${idx}_SERVER_IMAP_AUTH ${source_conf}`" ]
        then
            case ${MAIL_INSTALLED} in
                mail)
                    mecho "- WEBMAIL_${idx}_SERVER_IMAP_AUTH='md5'"
                    eval "WEBMAIL_${idx}_SERVER_IMAP_AUTH='md5'"
                    ;;
                vmail)
                    mecho "- WEBMAIL_${idx}_SERVER_IMAP_AUTH='login'"
                    eval "WEBMAIL_${idx}_SERVER_IMAP_AUTH='login'"
                    ;;
            esac

            added=1
        fi

        if [ -z "`grep ^WEBMAIL_${idx}_SERVER_IMAP_TYPE ${source_conf}`" ]
        then
            case ${MAIL_INSTALLED} in
                mail)
                    mecho "- WEBMAIL_${idx}_SERVER_IMAP_TYPE='uw'"
                    eval "WEBMAIL_${idx}_SERVER_IMAP_TYPE='uw'"
                    ;;
                vmail)
                    # set platform specific parameters
                    case ${EISFAIR_SYSTEM} in
                        eisfair-1)
                            # eisfair-1
                            mecho "- WEBMAIL_${idx}_SERVER_IMAP_TYPE='courier'"
                            eval "WEBMAIL_${idx}_SERVER_IMAP_TYPE=''courier'"
                            ;;
                        *)
                            # default to eisfair-2/eisxen-1
                            mecho "- WEBMAIL_${idx}_SERVER_IMAP_TYPE='dovecot'"
                            eval "WEBMAIL_${idx}_SERVER_IMAP_TYPE=''dovecot'"
                            ;;
                    esac
                    ;;
            esac
            added=1
        fi

        if [ -z "`grep ^WEBMAIL_${idx}_SERVER_SMTP_HOST ${source_conf}`" ]
        then
            mecho "- WEBMAIL_${idx}_SERVER_SMTP_HOST='localhost'"
            eval "WEBMAIL_${idx}_SERVER_SMTP_HOST='localhost'"
            added=1
        fi

        if [ -z "`grep ^WEBMAIL_${idx}_SERVER_SMTP_AUTH ${source_conf}`" ]
        then
            case ${MAIL_INSTALLED} in
                mail)
                    mecho "- WEBMAIL_${idx}_SERVER_SMTP_AUTH='md5'"
                    eval "WEBMAIL_${idx}_SERVER_SMTP_AUTH='md5'"
                    ;;
                vmail)
                    mecho "- WEBMAIL_${idx}_SERVER_SMTP_AUTH='login'"
                    eval "WEBMAIL_${idx}_SERVER_SMTP_AUTH='login'"
            esac
            added=1
        fi

        # v1.5.17 - upgrade to SM v1.4.20
        if [ -z "`grep ^WEBMAIL_${idx}_SERVER_DOMAIN_CHECK ${source_conf}`" ]
        then
            mecho "- WEBMAIL_${idx}_SERVER_DOMAIN_CHECK='yes'"
            eval "WEBMAIL_${idx}_SERVER_DOMAIN_CHECK='yes'"
            added=1
        fi

        if [ -z "`grep ^WEBMAIL_${idx}_SERVER_IMAP_TRANSPORT ${source_conf}`" ]
        then
            mecho "- WEBMAIL_${idx}_SERVER_IMAP_TRANSPORT='default'"
            eval "WEBMAIL_${idx}_SERVER_IMAP_TRANSPORT='default'"
            added=1
        fi

        if [ -z "`grep ^WEBMAIL_${idx}_SERVER_SMTP_TRANSPORT ${source_conf}`" ]
        then
            mecho "- WEBMAIL_${idx}_SERVER_SMTP_TRANSPORT='default'"
            eval "WEBMAIL_${idx}_SERVER_SMTP_TRANSPORT='default'"
            added=1
        fi

        eval globldap_n='$WEBMAIL_'${idx}'_GLOBADDR_LDAP_N'

        if [ ${globldap_n} -eq 0 ]
        then
            jmax=1
        else
            jmax=$globldap_n
        fi

        jdx=1
        while [ ${jdx} -le ${jmax} ]
        do
            if [ -z "`grep ^WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_ACTIVE ${source_conf}`" ]
            then
                mecho "- WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_ACTIVE='no'"
                eval "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_ACTIVE='no'"
                added=1
            fi

            if [ -z "`grep ^WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_INFO ${source_conf}`" ]
            then
                mecho "- WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_INFO='My LDAP directory'"
                eval "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_INFO='My LDAP directory'"
                added=1
            fi

            if [ -z "`grep ^WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_HOST ${source_conf}`" ]
            then
                mecho "- WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_HOST='localhost'"
                eval "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_HOST='localhost'"
                added=1
            fi

            if [ -z "`grep ^WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BASEDN ${source_conf}`" ]
            then
                mecho "- WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BASEDN='ou=contacts,ou=addressbook,dc=domain,dc=lan'"
                eval "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BASEDN='ou=contacts,ou=addressbook,dc=domain,dc=lan'"
                added=1
            fi

            if [ -z "`grep ^WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_AUTH ${source_conf}`" ]
            then
                mecho "- WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_AUTH='no'"
                eval "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_AUTH='no'"
                added=1
            fi

            if [ -z "`grep ^WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BINDDN ${source_conf}`" ]
            then
                mecho "- WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BINDDN='cn=ldapadmin,dc=domain,dc=lan'"
                eval "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BINDDN='cn=ldapadmin,dc=domain,dc=lan'"
                added=1
            fi

            if [ -z "`grep ^WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BINDPASS ${source_conf}`" ]
            then
                mecho "- WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BINDPASS='my-secret'"
                eval "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BINDPASS='my-secret'"
                added=1
            fi

            jdx=`expr ${jdx} + 1`
        done

        idx=`expr ${idx} + 1`
    done

    if [ ${added} -eq 1 ]
    then
        mecho -info "... read documentation for new parameter(s)!"
        anykey
    fi
}

#------------------------------------------------------------------------------
# delete variables
#------------------------------------------------------------------------------
delete_variables ()
{
    deleted=0
    mecho "deleting old parameters ..."

    for varname in "WEBMAIL_FOLDER_MOVE_FOLDER_TO_TRASH"
    do
        if [ ! -z "`grep \"^$varname\" $source_conf`" ]
        then
            mecho "- $varname"
            deleted=1
        fi
    done

    if [ $deleted -eq 1 ]
    then
        anykey
    fi
}

#------------------------------------------------------------------------------
# create new configuration
#------------------------------------------------------------------------------
create_config ()
{
    mecho "updating/creating configuration ..."
    {
        #------------------------------------------------------------------------------
        printgpl "webmail" "2002-11-24" "jed" "Copyright (c) 2001-`date +\"%Y\"` The Eisfair Team, team(at)eisfair(dot)org"
        #------------------------------------------------------------------------------
        printgroup "start settings"
        #------------------------------------------------------------------------------

        printvar "START_WEBMAIL" "start Webmail server: yes or no"

        #------------------------------------------------------------------------------
        printgroup "server settings"
        #------------------------------------------------------------------------------

        printvar "WEBMAIL_N" "number of webmail instances"
        echo

        if [ $WEBMAIL_N -eq 0 ]
        then
            imax=1
        else
            imax=$WEBMAIL_N
        fi

        idx=1
        while [ $idx -le $imax ]
        do
            printvar "WEBMAIL_${idx}_ACTIVE"                "${idx}. activate: yes or no"
            printvar "WEBMAIL_${idx}_DOCUMENT_ROOT"         "   document root"
            printvar "WEBMAIL_${idx}_SERVER_DOMAIN"         "   your mail domain"
            printvar "WEBMAIL_${idx}_SERVER_DOMAIN_CHECK"   "   check domain referal: yes or no"
            printvar "WEBMAIL_${idx}_SERVER_IMAP_HOST"      "   hostname of imap server"
            printvar "WEBMAIL_${idx}_SERVER_IMAP_AUTH"      "   auth type: digest, md5 or login"

            # set platform specific parameters
            case ${EISFAIR_SYSTEM} in
                eisfair-1)
                    # eisfair-1
                    printvar "WEBMAIL_${idx}_SERVER_IMAP_TYPE"      "   server type: uw or courier"
                    ;;
                *)
                    # default to eisfair-2/eisxen-1
                    printvar "WEBMAIL_${idx}_SERVER_IMAP_TYPE"      "   server type: uw or dovecot"
                    ;;
            esac

            printvar "WEBMAIL_${idx}_SERVER_IMAP_TRANSPORT" "   transport to use: default or tls"
            printvar "WEBMAIL_${idx}_SERVER_SMTP_HOST"      "   hostname of smtp server"
            printvar "WEBMAIL_${idx}_SERVER_SMTP_AUTH"      "   auth type: digest, md5, login, none"
            printvar "WEBMAIL_${idx}_SERVER_SMTP_TRANSPORT" "   transport to use: default or tls"

            #------------------------------------------------------------------------------
            printgroup "organization settings"
            #------------------------------------------------------------------------------

            printvar "WEBMAIL_${idx}_ORGA_NAME"          "${idx}. organization name"
            printvar "WEBMAIL_${idx}_ORGA_TITLE"         "   organization title"
            echo
            printvar "WEBMAIL_${idx}_ORGA_LOGO"          "   logo path"
            printvar "WEBMAIL_${idx}_ORGA_LOGO_WIDTH"    "   logo width"
            printvar "WEBMAIL_${idx}_ORGA_LOGO_HEIGHT"   "   logo height"
            echo
            printvar "WEBMAIL_${idx}_ORGA_SIGNOUT_PAGE"  "   signout page, empty means 'use default'"
            printvar "WEBMAIL_${idx}_ORGA_DEF_LANGUAGE"  "   default language"
            printvar "WEBMAIL_${idx}_ORGA_TOP_FRAME"     "   display in this frame"
            echo
            printvar "WEBMAIL_${idx}_ORGA_PROVIDER_NAME" "   provider name"
            printvar "WEBMAIL_${idx}_ORGA_PROVIDER_URL"  "   provider link"

            #------------------------------------------------------------------------------
            printgroup "folder settings"
            #------------------------------------------------------------------------------

            printvar "WEBMAIL_${idx}_FOLDER_MOVE_MSGS_TO_TRASH" "${idx}. move deleted messages to trash"
            printvar "WEBMAIL_${idx}_FOLDER_MOVE_MSGS_TO_SEND"  "   move sent messages to send folder"
            printvar "WEBMAIL_${idx}_FOLDER_MOVE_MSGS_TO_DRAFT" "   show move to draft folder option"
            echo
            printvar "WEBMAIL_${idx}_FOLDER_SPECIAL_LIST_FIRST" "   list special folder first"
            printvar "WEBMAIL_${idx}_FOLDER_SPECIAL_USE_COLOR"  "   use special folder color"
            echo
            printvar "WEBMAIL_${idx}_FOLDER_AUTO_EXPUNGE"       "   delete source msg after move"
            echo
            printvar "WEBMAIL_${idx}_FOLDER_UNSEEN_NOTIFY"      "   mail notification: 'none', 'inbox', 'all'"
            printvar "WEBMAIL_${idx}_FOLDER_UNSEEN_TYPE"        "   show 'unseen' or 'both' (unseen and total)"

            #------------------------------------------------------------------------------
            printgroup "general settings"
            #------------------------------------------------------------------------------

            printvar "WEBMAIL_${idx}_GENERAL_DEF_CHARSET"         "${idx}. used charset: iso-8859-1, koi8-r"
            printvar "WEBMAIL_${idx}_GENERAL_LEFT_FRAME_WIDTH"    "   width of left frame"
            printvar "WEBMAIL_${idx}_GENERAL_ALLOW_PRIORITY_USE"  "   allow setting of mail priority"
            printvar "WEBMAIL_${idx}_GENERAL_ALLOW_RECEIPTS_USE"  "   allow request of receipts"
            printvar "WEBMAIL_${idx}_GENERAL_ALLOW_IDENTITY_EDIT" "   allow editing of identity data"

            #------------------------------------------------------------------------------
            printgroup "themes settings"
            #------------------------------------------------------------------------------

            printvar "WEBMAIL_${idx}_THEMES_USE_ALL" "${idx}. yes - take all, no - take individual"
            printvar "WEBMAIL_${idx}_THEMES_DEFAULT" "   default theme number"
            echo
            printvar "WEBMAIL_${idx}_THEMES_N"       "   number of individual themes"

            eval themes_n='$WEBMAIL_'${idx}'_THEMES_N'

            if [ "${themes_n}" = "" ]
            then
                themes_n=0
            fi

            if [ $themes_n -eq 0 ]
            then
                jmax=4
            else
                jmax=$themes_n
            fi

            jdx=1
            while [ $jdx -le $jmax ]
            do
                printvar "WEBMAIL_${idx}_THEMES_${jdx}_FILENAME" "   ${jdx}. theme"
                jdx=`expr $jdx + 1`
            done

            #------------------------------------------------------------------------------
            printgroup "plugins settings"
            #------------------------------------------------------------------------------

            printvar "WEBMAIL_${idx}_PLUGINS_USE_ALL" "${idx}. yes - take all, no - take individual"
            echo
            printvar "WEBMAIL_${idx}_PLUGINS_N"       "   number of individual plugins"

            eval plugins_n='$WEBMAIL_'${idx}'_PLUGINS_N'

            if [ "${plugins_n}" = "" ]
            then
                plugins_n=0
            fi

            if [ $plugins_n -eq 0 ]
            then
                jmax=7
            else
                jmax=$plugins_n
            fi

            jdx=1
            while [ $jdx -le $jmax ]
            do
                printvar "WEBMAIL_${idx}_PLUGINS_${jdx}_DIRNAME" "   ${jdx}. plugin"
                jdx=`expr $jdx + 1`
            done

            echo
            printvar "WEBMAIL_${idx}_PLUGINS_ADMINUSER" "   admin user for administration plugin"

            #------------------------------------------------------------------------------
            printgroup "global address books"
            #------------------------------------------------------------------------------

            printvar "WEBMAIL_${idx}_GLOBADDR_FILE_WRITEABLE" "${idx}. allow editing of addressbook entries"
            echo
            printvar "WEBMAIL_${idx}_GLOBADDR_FILE_N"         "   number of addressbook entries"

            eval globaddr_n='$WEBMAIL_'${idx}'_GLOBADDR_FILE_N'

            if [ ${globaddr_n} -eq 0 ]
            then
                jmax=1
            else
                jmax=${globaddr_n}
            fi

            jdx=1
            while [ ${jdx} -le ${jmax} ]
            do
                printvar "WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_NICKNAME"  "   ${jdx}. nickname"
                printvar "WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_FIRSTNAME" "      firstname"
                printvar "WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_LASTNAME"  "      lastname"
                printvar "WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_EMAIL"     "      email address"
                printvar "WEBMAIL_${idx}_GLOBADDR_FILE_${jdx}_INFO"      "      information"
                jdx=`expr $jdx + 1`
            done

            echo
            echo "#------------------------------------------------------------------------------"
            echo

            printvar "WEBMAIL_${idx}_GLOBADDR_LDAP_N"         "   number of ldap addressbooks"

            eval globldap_n='$WEBMAIL_'${idx}'_GLOBADDR_LDAP_N'

            if [ ${globldap_n} -eq 0 ]
            then
                jmax=1
            else
                jmax=${globldap_n}
            fi

            jdx=1
            while [ ${jdx} -le ${jmax} ]
            do
                printvar "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_ACTIVE"    "   activate ldap addressbook: yes or no"
                printvar "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_INFO"      "   description of ldap addressbook"
                printvar "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_HOST"      "   hostname of ldap server"
                printvar "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BASEDN"    "   base-dn of ldap addressbook"
                printvar "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_AUTH"      "   require authentication: yes or no"
                printvar "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BINDDN"    "   bind-dn for ldap authentication"
                printvar "WEBMAIL_${idx}_GLOBADDR_LDAP_${jdx}_BINDPASS"  "   bind-password for ldap"
                printvar ""                                              "   authentication"
                jdx=`expr $jdx + 1`
            done

            #------------------------------------------------------------------------------
            printgroup "others"
            #------------------------------------------------------------------------------

            printvar "WEBMAIL_ATTACH_CRON_SCHEDULE" "cron configuration string"
            echo
            printvar "WEBMAIL_LOG_COUNT"            "number of log files to save"
            printvar "WEBMAIL_LOG_INTERVAL"         "interval: daily, weekly, monthly"

            #------------------------------------------------------------------------------
            printend
            #------------------------------------------------------------------------------

            idx=`expr ${idx} + 1`
        done
    } > ${generate_conf}
}

#------------------------------------------------------------------------------
# old: merge global addresses
#------------------------------------------------------------------------------
old_merge_global_addresses ()
{
    if [ -f $webmail_globaddr ]
    then
        if [ -f $webmail_global_addresses.csv ]
        then
            /var/install/bin/backup-file -quiet "$webmail_global_addresses.csv" backup

            mecho "merging $webmail_globaddr and $webmail_global_addresses.csv files ..."

            # convert to unix file format
            dtou $webmail_global_addresses.csv
        else
            mecho "importing $webmail_globaddr file ..."

            {
                # target file does not exist - export data
                echo "#"
                echo "# file created by $0 on `date`"
                echo "#"
                echo "#nick,firstname,lastname,email address,comment"
            } > $webmail_global_addresses.csv
        fi

        # convert to unix file format
        dtou $webmail_globaddr

        OLDIFS=$IFS
        IFS=\|
        while read line
        do
            # delete leading spaces
            line=`trim_spaces "$line"`

            if [ "$line" != "" ]
            then
                # line not empty
                echo $line|grep "^#" > /dev/null

                if [ $? -eq 1 ]
                then
                    # carmen|Carmen|Kuller|carmen.kuller@testnet.lan|
                    # no comment found - process line
                    # extract information and remove leading/trailing spaces
                    set -- $line
                    nick=`trim_spaces "$1"`
                    firstname=`trim_spaces "$2"`
                    lastname=`trim_spaces "$3"`
                    address=`trim_spaces "$4"`
                    info=`trim_spaces "$5"`

                    # carmen,Carmen,Kuller,carmen.kuller@testnet.lan,
                    # check if nick name already exists
                    jdx=1
                    grep -i -q "^ *${nick} *," $webmail_global_addresses.csv

                    if [ $? -ne 0 ]
                    then
                        # add entry
                        mecho "- adding '$nick' ..."

                        echo "${nick},${firstname},${lastname},${address},${info}" >> $webmail_global_addresses.csv
                    else
                        # skip entry
                        mecho "- skipping '$nick' - entry already exists ..."
                    fi
                fi
            fi
        done < $webmail_globaddr

        IFS=$OLDIFS
    fi
}

#------------------------------------------------------------------------------
# merge global addresses
# $1 - webmail instance
#------------------------------------------------------------------------------
merge_global_addresses ()
{
    wm_nbr=$1
    if [ $wm_nbr -ne 0 -a $wm_nbr -le $WEBMAIL_N ]
    then
        if [ -f ${webmail_globaddr}-${wm_nbr} ]
        then
            if [ -f ${webmail_global_addresses}-${wm_nbr}.csv ]
            then
                /var/install/bin/backup-file -quiet "${webmail_global_addresses}-${wm_nbr}.csv" backup

                mecho "merging ${webmail_globaddr}.${wm_nbr} and ${webmail_global_addresses}-${wm_nbr}.csv files ..."

                # convert to unix file format
                dtou ${webmail_global_addresses}-${wm_nbr}.csv
            else
                mecho "importing ${webmail_globaddr}-${wm_nbr} file ..."

                {
                    # target file does not exist - export data
                    echo "#"
                    echo "# file created by $0 on `date`"
                    echo "#"
                    echo "#nick,firstname,lastname,email address,comment"
                } > ${webmail_global_addresses}-${wm_nbr}.csv
            fi

            # convert to unix file format
            dtou ${webmail_globaddr}-${wm_nbr}

            OLDIFS=$IFS
            IFS=\|
            while read line
            do
                # delete leading spaces
                line=`trim_spaces "$line"`

                if [ "$line" != "" ]
                then
                    # line not empty
                    echo $line|grep "^#" > /dev/null

                    if [ $? -eq 1 ]
                    then
                        # carmen|Carmen|Kuller|carmen.kuller@testnet.lan|
                        # no comment found - process line
                        # extract information and remove leading/trailing spaces
                        set -- $line
                        nick=`trim_spaces "$1"`
                        firstname=`trim_spaces "$2"`
                        lastname=`trim_spaces "$3"`
                        address=`trim_spaces "$4"`
                        info=`trim_spaces "$5"`

                        # carmen,Carmen,Kuller,carmen.kuller@testnet.lan,
                        # check if nick name already exists
                        jdx=1
                        grep -i -q "^ *${nick} *," ${webmail_global_addresses}-${wm_nbr}.csv

                        if [ $? -ne 0 ]
                        then
                            # add entry
                            mecho "- adding '$nick' ..."

                            echo "${nick},${firstname},${lastname},${address},${info}" >> ${webmail_global_addresses}-${wm_nbr}.csv
                        else
                            # skip entry
                            mecho "- skipping '$nick' - entry already exists ..."
                        fi
                    fi
                fi
            done < ${webmail_globaddr}-${wm_nbr}

            IFS=$OLDIFS
        fi
    else
        mecho -error "Invalid instance number '${wm_nbr}' given, export aborted!"
    fi
}

#------------------------------------------------------------------------------
# import global addresses
#------------------------------------------------------------------------------
old_import_global_addresses ()
{
    if [ -f $webmail_global_addresses.csv ]
    then
        mecho "importing $webmail_global_addresses.csv file ..."

        # convert to unix file format
        dtou $webmail_global_addresses.csv

        OLDIFS=$IFS
        IFS=,
        # set offset
        idx=$WEBMAIL_GLOBADDR_N
        while read line
        do
            # delete leading spaces
            line=`trim_spaces "$line"`

            if [ "$line" != "" ]
            then
                # line not empty
                echo $line|grep "^#" > /dev/null

                if [ $? -eq 1 ]
                then
                    # no comment found - process line
                    # extract information and remove leading/trailing spaces
                    set $line
                    nick=`trim_spaces "$1"`
                    firstname=`trim_spaces "$2"`
                    lastname=`trim_spaces "$3"`
                    address=`trim_spaces "$4"`
                    info=`trim_spaces "$5"`

                    # check if nick name already exists
                    jdx=1
                    foundflag=0
                    testnick1=`echo $nick|tr 'A-Z' 'a-z'`
                    while [ $jdx -le $WEBMAIL_GLOBADDR_N ]
                    do
                        eval testnick2='$WEBMAIL_GLOBADDR_'$jdx'_NICKNAME'

                        testnick2=`trim_spaces "$testnick2"|tr 'A-Z' 'a-z'`

                        if [ "$testnick1" = "$testnick2" ]
                        then
                            foundflag=1
                            break
                        fi

                        jdx=`expr $jdx + 1`
                    done

                    if [ $foundflag -eq 0 ]
                    then
                        # add entry
                        idx=`expr $idx + 1`

                        eval 'WEBMAIL_GLOBADDR_'$idx'_NICKNAME'="$nick"
                        eval 'WEBMAIL_GLOBADDR_'$idx'_FIRSTNAME'="$firstname"
                        eval 'WEBMAIL_GLOBADDR_'$idx'_LASTNAME'="$lastname"
                        eval 'WEBMAIL_GLOBADDR_'$idx'_EMAIL'="$address"
                        eval 'WEBMAIL_GLOBADDR_'$idx'_INFO'="\'$info\'"

                        mecho "- adding '$nick' ..."

                        # increment counter
                        WEBMAIL_GLOBADDR_N=$idx
                    else
                        # skip entry
                        mecho "- skipping '$nick' - entry already exists ..."
                    fi
                fi
            fi
        done < $webmail_global_addresses.csv

        IFS=$OLDIFS
    fi
}

#------------------------------------------------------------------------------
# import global addresses
# $1 - webmail instance
#------------------------------------------------------------------------------
import_global_addresses ()
{
    wm_nbr=$1
    if [ $wm_nbr -ne 0 -a $wm_nbr -le $WEBMAIL_N ]
    then
        if [ -f ${webmail_global_addresses}-${wm_nbr}.csv ]
        then
            eval globaddr_n='$WEBMAIL_'${wm_nbr}'_GLOBADDR_N'

            if [ "${globaddr_n}" = "" ]
            then
                # new format since v1.4.10
                eval globaddr_n='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_N'
                extstr='FILE_'
            else
                # previous format since v1.2.11
                extstr=''
            fi

            mecho "importing ${webmail_global_addresses}-${wm_nbr}.csv file ..."

            # convert to unix file format
            dtou ${webmail_global_addresses}-${wm_nbr}.csv

            OLDIFS=$IFS
            IFS=,
            # set offset
            eval idx='$WEBMAIL_'${wm_nbr}'_GLOBADDR_'${extstr}'N'
            while read line
            do
                # delete leading spaces
                line=`trim_spaces "$line"`

                if [ "$line" != "" ]
                then
                    # line not empty
                    echo $line | grep -q "^#"

                    if [ $? -eq 1 ]
                    then
                        # no comment found - process line
                        # extract information and remove leading/trailing spaces
                        set $line
                        nick=`trim_spaces "$1"`
                        firstname=`trim_spaces "$2"`
                        lastname=`trim_spaces "$3"`
                        address=`trim_spaces "$4"`
                        info=`trim_spaces "$5"`

                        # check if nick name already exists
                        jdx=1
                        foundflag=0
                        testnick1=`echo $nick|tr 'A-Z' 'a-z'`
                        while [ $jdx -le $globaddr_n ]
                        do
                            eval testnick2='$WEBMAIL_'${wm_nbr}'_GLOBADDR_'${extstr}${jdx}'_NICKNAME'

                            testnick2=`trim_spaces "$testnick2"|tr 'A-Z' 'a-z'`

                            if [ "$testnick1" = "$testnick2" ]
                            then
                                foundflag=1
                                break
                            fi

                            jdx=`expr $jdx + 1`
                        done

                        if [ $foundflag -eq 0 ]
                        then
                            # add entry
                            idx=`expr $idx + 1`

                            eval 'WEBMAIL_'${wm_nbr}'_GLOBADDR_'${extstr}${idx}'_NICKNAME'="${nick}"
                            eval 'WEBMAIL_'${wm_nbr}'_GLOBADDR_'${extstr}${idx}'_FIRSTNAME'="${firstname}"
                            eval 'WEBMAIL_'${wm_nbr}'_GLOBADDR_'${extstr}${idx}'_LASTNAME'="${lastname}"
                            eval 'WEBMAIL_'${wm_nbr}'_GLOBADDR_'${extstr}${idx}'_EMAIL'="${address}"
                            eval 'WEBMAIL_'${wm_nbr}'_GLOBADDR_'${extstr}${idx}'_INFO'="'${info}'"

                            mecho "- adding '$nick' ..."

                            # increment counter
                            eval WEBMAIL_${wm_nbr}_GLOBADDR_${extstr}N=${idx}
                        else
                            # skip entry
                            mecho "- skipping '${nick}' - entry already exists ..."
                        fi
                    fi
                fi
            done < ${webmail_global_addresses}-${wm_nbr}.csv

            IFS=$OLDIFS
        fi
    else
        mecho -error "Invalid instance number '${wm_nbr}' given, export aborted!"
    fi
}

#------------------------------------------------------------------------------
# old: export global addressbook
#------------------------------------------------------------------------------
old_export_global_addresses ()
{
    if [ $WEBMAIL_GLOBADDR_N -gt 0 ]
    then
        if [ -f $webmail_global_addresses.csv ]
        then
            /var/install/bin/backup-file -quiet "${webmail_global_addresses}.csv" backup
            rm ${webmail_global_addresses}.csv
        fi

        mecho "exporting $webmail_global_addresses.csv file ..."

        {
            # target file does not exist - export data
            echo "#"
            echo "# file created by $0 on `date`"
            echo "#"
            echo "#nick,firstname,lastname,email address,comment"
        } > $webmail_global_addresses.csv

        idx=1
        while [ $idx -le $WEBMAIL_GLOBADDR_N ]
        do
            eval nick='$WEBMAIL_GLOBADDR_'$idx'_NICKNAME'
            eval firstname='$WEBMAIL_GLOBADDR_'$idx'_FIRSTNAME'
            eval lastname='$WEBMAIL_GLOBADDR_'$idx'_LASTNAME'
            eval address='$WEBMAIL_GLOBADDR_'$idx'_EMAIL'
            eval info='$WEBMAIL_GLOBADDR_'$idx'_INFO'

            echo "$nick,$firstname,$lastname,$address,$info" >> $webmail_global_addresses.csv
            mecho "- adding '$nick' ..."
            idx=`expr $idx + 1`
        done

        mecho "Global addresses exported to $webmail_global_addresses.csv."
    fi
}

#------------------------------------------------------------------------------
# export global addressbook
# $1 - webmail instance
#------------------------------------------------------------------------------
export_global_addresses ()
{
    wm_nbr=$1
    if [ $wm_nbr -ne 0 -a $wm_nbr -le $WEBMAIL_N ]
    then
        eval globaddr_n='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_N'

        if [ $globaddr_n -gt 0 ]
        then
            if [ -f ${webmail_global_addresses}-${wm_nbr}.csv ]
            then
                /var/install/bin/backup-file -quiet "${webmail_global_addresses}-${idx}.csv" backup
                rm ${webmail_global_addresses}-${wm_nbr}.csv
            fi

            mecho "exporting ${webmail_global_addresses}-${wm_nbr}.csv file ..."

            {
                # target file does not exist - export data
                echo "#"
                echo "# file created by $0 on `date`"
                echo "#"
                echo "#nick,firstname,lastname,email address,comment"
            } > ${webmail_global_addresses}-${wm_nbr}.csv

            idx=1
            while [ ${idx} -le ${globaddr_n} ]
            do
                eval nick='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_'${idx}'_NICKNAME'
                eval firstname='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_'${idx}'_FIRSTNAME'
                eval lastname='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_'${idx}'_LASTNAME'
                eval address='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_'${idx}'_EMAIL'
                eval info='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_'${idx}'_INFO'

                echo "$nick,$firstname,$lastname,$address,$info" >> ${webmail_global_addresses}-${wm_nbr}.csv

                mecho "- adding '${nick}' ..."
                idx=`expr ${idx} + 1`
            done

            mecho "Global addresses exported to ${webmail_global_addresses}-${wm_nbr}.csv."
        fi
    else
        mecho -error "Invalid instance number '${wm_nbr}' given, export aborted!"
    fi
}

#------------------------------------------------------------------------------
# rename 'old' program files to 'new' multi instance format
#------------------------------------------------------------------------------
rename_old_files ()
{
    ls $conf_tmpdir/??_??-filelist.txt > /dev/null 2> /dev/null

    if [ $? -eq 0 ]
    then
        mecho "renaming files ..."
        for FNAME in $conf_tmpdir/??_??-filelist.txt
        do
            new_name=`echo "$FNAME" | sed 's/.txt$//g'`

            mecho "- $FNAME -> $new_name-1.txt"
            mv "$FNAME" "$new_name-1.txt" 
        done

        if [ -f "$webmail_global_addresses.csv" ]
        then
            mecho "- $webmail_global_addresses.csv -> $webmail_global_addresses-1.csv"
            mv "$webmail_global_addresses.csv" "$webmail_global_addresses-1.csv"
        fi

        if [ -f "$webmail_globaddr" ]
        then
            mecho "- $webmail_globaddr -> ${webmail_globaddr}-1"
            mv "$webmail_globaddr" "${webmail_globaddr}-1"
        fi
    fi
}

#==============================================================================
# main
#==============================================================================

#testroot=/soft/jedwebmail
 testroot=''

# set platform specific parameters
case ${EISFAIR_SYSTEM} in
    eisfair-1)
        # eisfair-1
        webmail_path=${testroot}/var/webmail
        ;;
    *)
        # default to eisfair-2/eisxen-1
        webmail_path=${testroot}/data/packages/webmail
        ;;
esac

### set file names ###
mailfile=$testroot/etc/config.d/mail
vmailfile=$testroot/etc/config.d/vmail
installfile=$testroot/var/run/webmail.install
webmailfile=$testroot/etc/config.d/webmail
webmail_globaddr=${webmail_path}/data/global.abook
webmail_global_addresses=${webmail_path}/global_addresses
conf_tmpdir=${webmail_path}

# setting defaults
source_conf=$installfile
generate_conf=$webmailfile

goflag=0

if check_installed_mail -quiet
then
    # mail
    MAIL_INSTALLED='mail'
elif check_installed_vmail -quiet
then
    # vmail
    MAIL_INSTALLED='vmail'
else
    # no local mail or vmail package installed
    MAIL_INSTALLED='none'
fi

case "$1"
in
    merge)
        # merge global.abook and global_addresses.csv
        if [ -f $webmail_global_addresses ]
        then
            /var/install/bin/backup-file -quiet "$webmail_global_addresses" backup
        fi

        if [ "$WEBMAIL_N" != "" ]
        then
            idx=1
            while [ $idx -le $WEBMAIL_N ]
            do
                merge_global_addresses $idx
                idx=`expr $idx + 1`
            done
        else
            # old configuration found -> old merge
            old_merge_global_addresses
        fi
        goflag=0
        ;;

    import)
        # import webmail global address list
        ls ${webmail_global_addresses}*.csv > /dev/null 2> /dev/null

        if [ $? -eq 0 ]
        then
            /var/install/bin/backup-file -quiet "$webmailfile"

            source_conf=$conf_tmpdir/`basename $webmailfile`.tmp
            cp $webmailfile $source_conf
            goflag=1
        fi
        ;;

    export)
        # export webmail global address list
        source_conf=$webmailfile
        . $source_conf

        if [ "$WEBMAIL_N" != "" ]
        then
            idx=1
            while [ $idx -le $WEBMAIL_N ]
            do
                export_global_addresses $idx
                idx=`expr $idx + 1`
            done
        else
            # old configuration found -> old export
            old_export_global_addresses
        fi
        goflag=0
        ;;

    update)
        goflag=1
        ;;

    test)
      # source_conf=$webmailfile.new
        source_conf=$webmailfile

        generate_conf=${webmailfile}.test
        goflag=1
        ;;

    *)
        mecho
        mecho "Use one of the following options:"
        mecho
        mecho "  webmail-update.sh [test]   - the file $webmailfile.new will be read and a test configuration"
        mecho "                               will be written to the file $testroot/etc/config.d/mk_webmail.test."
        mecho
        mecho "  webmail-update.sh [update] - the file $webmailfile.import will be read, the configuration will"
        mecho "                               be checked and an updated mail configuration file will be written."
        mecho
        goflag=0
        ;;
esac

if [ $goflag -eq 1 ]
then
    if [ -f $source_conf ]
    then
        # previous configuration file exists
        mecho "previous configuration found ..."
        . $source_conf

        rename_old_files

        if [ "$WEBMAIL_N" != "" ]
        then
            wmidx=1
            while [ $wmidx -le $WEBMAIL_N ]
            do
                merge_global_addresses $wmidx
                import_global_addresses $wmidx

                wmidx=`expr $wmidx + 1`
            done
        else
            # old configuration found -> old merge/import
            old_merge_global_addresses
            old_import_global_addresses
        fi

        pre_add_variables
        rename_variables

        modify_variables
        add_variables
        delete_variables

        create_config

        mecho "finished."
    else
        mecho -error "no configuration $source_conf found - exiting."
    fi
fi

#==============================================================================
# end
#==============================================================================
