#!/bin/sh
#----------------------------------------------------------------------------------------
# /var/install/config.d/webmail.sh - configuration generator script for Webmail server
#
# Copyright (c) 2001-2011 The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:    2002-11-29 jed
# Last Update: $Id: webmail.sh 28811 2011-09-11 09:40:07Z jed $
#
# Parameters:
#   webmail.sh            - generates all configuration files
#   webmail.sh removecron - remove cronjob
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------------------

# read eislib
. /var/install/include/eislib
. /var/install/include/check-eisfair-version
. /var/install/include/jedlib

#exec 2>/tmp/webmail-trace-$$.log
#set -x
#testroot=/soft/jedwebmail                                            # only for testing
testroot=""

pgmname=`basename $0`
tty=`tty`

# set platform specific parameters
case ${EISFAIR_SYSTEM} in
    eisfair-1)
        # eisfair-1
        crontab_path=${testroot}/var/cron/etc/root
        webmail_path=${testroot}/var/webmail
        webmail_apache_user="wwwrun"
        webmail_apache_group="nogroup"
        ;;
    *)
        # eisfair-2
        crontab_path=${testroot}/etc/cron/root
        webmail_path=${testroot}/data/packages/webmail
        webmail_apache_user="www-data"
        webmail_apache_group="www-data"
        ;;
esac

### set path names ###
webmail_data_path=${webmail_path}/data
webmail_attach_path=${webmail_path}/attach
webmail_log_path=${webmail_path}/log

### set file names ###
mailfile=$testroot/etc/config.d/mail
php5file=$testroot/etc/config.d/apache2_php5
vmailfile=$testroot/etc/config.d/vmail
webmailfile=$testroot/etc/config.d/webmail
configlog_file=$webmail_log_path/webmail-configlog
crontab_file=$crontab_path/webmail
services_file=$testroot/etc/services
generate_logrotate=$testroot/etc/logrotate.d/webmail
generate_globaddr=$webmail_data_path/global.abook
webmail_vmail_config_file=${testroot}/var/install/config.d/webmail_vmail.sh

docroot_filelist=${webmail_path}/wm-filelist.txt
docroot_addlist=${webmail_path}/wm-docroot.lst
docroot_dellist=${webmail_path}/wm-docroot.del
docroot_tmplist=/tmp/wm-docroot.$$

### other parameters ###
webmail_version="v1.6.2-24"

### load configuration ###
. $webmailfile
chmod 600 $webmailfile

#----------------------------------------------------------------------------------------
# check if mail has been enabled
#----------------------------------------------------------------------------------------
check_installed_mail ()
{
    retval=1

    if [ -f ${mailfile} ]
    then
        # mail installed
        . ${mailfile}

        if [ "${START_MAIL}" = "yes" ]
        then
            # mail activated
            if [ "${1}" != "-quiet" ]
            then
                mecho "mail has been enabled ..."
            fi
            retval=0
        else
            # mail deactivated
            if [ "${1}" != "-quiet" ]
            then
                mecho -warn "mail has been disabled ..."
            fi
        fi
    fi

    return ${retval}
}

#----------------------------------------------------------------------------------------
# check if vmail has been enabled
#----------------------------------------------------------------------------------------
check_installed_vmail ()
{
    retval=1

    if [ -f ${vmailfile} ]
    then
        # mail installed
        . ${vmailfile}

        if [ "${START_VMAIL}" = "yes" ]
        then
            # vmail activated
            if [ "${1}" != "-quiet" ]
            then
                mecho "vmail has been enabled ..."
            fi
            retval=0
        else
            # vmail deactivated
            if [ "${1}" != "-quiet" ]
            then
                mecho -warn "vmail has been disabled ..."
            fi
        fi
    fi

    return ${retval}
}

#----------------------------------------------------------------------------------------
# check if vmail-plugin has been enabled
#----------------------------------------------------------------------------------------
check_installed_vmail_plugin ()
{
    retval=1

    # check webmail_vmail version
    installed_version=`/var/install/bin/check-version webmail_vmail 0.1.0`

    case ${installed_version} in
        not-installed|new ) # wrong version
            retval=1
            ;;
        * ) # installed / old
            retval=0
            ;;
    esac

    if [ "${1}" != "-quiet" ]
    then
        if [ ${retval} -eq 0 ]
        then
            mecho "webmail_vmail has been installed ..."
        else
            mecho "webmail_vmail has not been installed ..."
        fi
    fi

    return ${retval}
}

#----------------------------------------------------------------------------------------
# check if php_ldap has been enabled
#----------------------------------------------------------------------------------------
check_active_php_ldap ()
{
    retval=1

    if [ -f ${php5file} ]
    then
        # mail installed
        . ${php5file}

        if [ "${PHP5_EXT_LDAP}" = "yes" ]
        then
            # ldap support activated
            if [ "${1}" != "-quiet" ]
            then
                mecho "php-ldap has been enabled ..."
            fi
            retval=0
        else
            # ldap support deactivated
            if [ "${1}" != "-quiet" ]
            then
                mecho -warn "php-ldap has been disabled ..."
            fi
        fi
    fi

    return ${retval}
}

#----------------------------------------------------------------------------------------
# creating squirrelmail configuration
#----------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------
# sub function: process complete themes list
# $1 - webmail instance
#----------------------------------------------------------------------------------------
process_all_themes ()
{
    wmnbr=$1
    eval wmdoc_root='$WEBMAIL_'${wmnbr}'_DOCUMENT_ROOT'
    wmthemes_path="$wmdoc_root/themes"

    cd "$wmthemes_path"

    # print default theme
    echo "\$theme[0]['PATH'] = SM_PATH . 'themes/default_theme.php';"
    echo "\$theme[0]['NAME'] = '_Default Theme';"

    idx=1
    for FNAME in `ls *.php|grep -i -v -e "index.php" -e "default_theme.php"`
    do
        # delete TABS, spaces, single and double quotes from theme title
        title=`grep -i "name:" $FNAME|cut -d: -f2|sed -n '1p'|tr \' ' '|sed -e 's/\*\///g' -e 's/^[\t "]*//g' -e 's/[\t "]$//g'` > /dev/null
        title=`echo $title` > /dev/null

        if [ "$title" = "" ]
        then
            # take file name as title
            title=`echo $FNAME|cut -d. -f1` > /dev/null
        fi

        # print theme name
        echo "\$theme[$idx]['PATH'] = SM_PATH . 'themes/$FNAME';"
        echo "\$theme[$idx]['NAME'] = '$title';"

        idx=`expr $idx + 1`
    done
}

#----------------------------------------------------------------------------------------
# sub function: process individual themes list
# $1 - webmail instance
#----------------------------------------------------------------------------------------
process_individual_themes ()
{
    wmnbr=$1
    eval wmdoc_root='$WEBMAIL_'${wmnbr}'_DOCUMENT_ROOT'
    eval wmthemes_n='$WEBMAIL_'${wmnbr}'_THEMES_N'
    wmthemes_path="$wmdoc_root/themes"

    cd "$wmthemes_path"

    idx=1
    count=0
    while [ $idx -le $wmthemes_n  ]
    do
        eval FNAME='$WEBMAIL_'${wmnbr}'_THEMES_'$idx'_FILENAME'

        if [ -f $FNAME ]
        then
            # delete TABS, spaces, single and double quotes from theme title
            title=`grep -i "name:" $FNAME|cut -d: -f2|sed -n '1p'|tr -s "\'\r\n" '   '|sed -e 's/\*\///g' -e 's/^[\t "]*//g' -e 's/[\t "]$//g'` > /dev/null
            title=`echo $title` > /dev/null

            if [ "$title" = "" ]
            then
                # take file name as title
                title=`echo $FNAME|cut -d. -f1` > /dev/null
            fi

            # if default theme than prefix with '_' to be sorted to the beginning
            if [ "$FNAME" = "default_theme.php" ]
            then
                title=_$title
            fi

            # print theme name
            echo "\$theme[$count]['PATH'] = SM_PATH . 'themes/$FNAME';"
            echo "\$theme[$count]['NAME'] = '$title';"

            count=`expr $count + 1`
        else
            write_to_config_log -warn "Theme \"$FNAME\" will be ignored because it could not be found!"
        fi

        idx=`expr $idx + 1`
    done
}

#----------------------------------------------------------------------------------------
# sub function: process complete plugins list
# $1 - webmail instance
#----------------------------------------------------------------------------------------
process_all_plugins ()
{
    wmnbr=${1}
    eval wmdoc_root='$WEBMAIL_'${wmnbr}'_DOCUMENT_ROOT'
    eval wmimap_type='$WEBMAIL_'${wmnbr}'_SERVER_IMAP_TYPE'
    wmplugins_path="${wmdoc_root}/plugins"

    cd "${wmplugins_path}"

    idx=0
    for DNAME in `ls -p|grep \/|cut -d\/ -f1`
    do
        case ${DNAME} in
            'uw_folders')
                if [ "${wmimap_type}" = 'uw' ]
                then
                    # add default plugin uw_folders
                    if [ -d "${wmplugins_path}/uw_folders" ]
                    then
                        # print plugin directory name
                        echo "\$plugins[${idx}] = '${DNAME}';"

                        idx=`expr ${idx} + 1`
                    else
                        write_to_config_log -warn "Plugin \"uw_folders\" could not be found therefore UW shared and public folders won't be supported!"
                    fi
                fi
                ;;
            'vmail_maildrop')
                if [ "${wmimap_type}" = 'courier' ]
                then
                    # add default plugin uw_folders
                    if [ -d "${wmplugins_path}/vmail_maildrop" ]
                    then
                        # print plugin directory name
                        echo "\$plugins[${idx}] = '${DNAME}';"

                        idx=`expr ${idx} + 1`
                    else
                        write_to_config_log -warn "Plugin \"vmail_maildrop\" could not be found therefore vmail features won't be supported!"
                    fi
                fi
                ;;
            *)
                # print plugin directory name
                echo "\$plugins[${idx}] = '${DNAME}';"

                idx=`expr ${idx} + 1`
                ;;
        esac
    done
}

#----------------------------------------------------------------------------------------
# sub function: process individual themes list
# $1 - webmail instance
#----------------------------------------------------------------------------------------
process_individual_plugins ()
{
    wmnbr=$1
    eval wmdoc_root='$WEBMAIL_'${wmnbr}'_DOCUMENT_ROOT'
    eval wmimap_type='$WEBMAIL_'${wmnbr}'_SERVER_IMAP_TYPE'
    eval wmplugins_n='$WEBMAIL_'${wmnbr}'_PLUGINS_N'
    wmplugins_path="${wmdoc_root}/plugins"

    cd "${wmplugins_path}"

    idx=1
    count=0
    uwflag=0
    vmflag=0
    while [ ${idx} -le ${wmplugins_n} ]
    do
        eval DNAME='$WEBMAIL_'${wmnbr}'_PLUGINS_'${idx}'_DIRNAME'

        if [ -d ${wmplugins_path}/${DNAME} ]
        then
            case ${DNAME} in
                'uw_folders')
                    # check if uw_folders plugin has already been added
                    uwflag=1
                    ;;
                'vmail_maildrop')
                    # check if webmail_vmail plugin has already been added
                    vmflag=1
                    ;;
            esac

            # print plugin directory name
            echo "\$plugins[${count}] = '${DNAME}';"

            count=`expr ${count} + 1`
        else
            write_to_config_log -warn "Plugin \"${DNAME}\" will be ignored because it could not be found!"
        fi

        idx=`expr ${idx} + 1`
    done

    case ${wmimap_type} in
        uw)
            if [ ${uwflag} -eq 0 ]
            then
                # add default plugin uw_folders
                if [ -d "${wmplugins_path}/uw_folders" ]
                then
                    # print plugin directory name
                    echo "\$plugins[${count}] = 'uw_folders';"
                else
                    write_to_config_log -warn "Plugin \"uw_folders\" could not be found therefore UW shared and public folders won't be supported!"
                fi
            fi
            ;;
        courier)
            if [ ${vmflag} -eq 0 ]
            then
                # add default plugin vmail_maildrop
                if [ -d "${wmplugins_path}/vmail_maildrop" ]
                then
                    # print plugin directory name
                    echo "\$plugins[${count}] = 'vmail_maildrop';"
                else
                    write_to_config_log -warn "Plugin \"vmail_maildrop\" could not be found therefore vmail features won't be supported!"
                fi
            fi
            ;;
    esac
}

#----------------------------------------------------------------------------------------
# get smtp port
# input : $1 - smtp port string
# return: smtp tcp-port number
#----------------------------------------------------------------------------------------
get_smtp_port ()
{
    smtp_str=$1

    if [ "$smtp_str" != "" ]
    then
        if is_numeric $smtp_str
        then
            # numeric value is ok
            smtp_nbr=$smtp_str
        else
            # non-numeric value
            smtp_nbr=`cat $services_file|tr -s ' \011\/' ':'|grep -E "^${smtp_str}:[0-9]+:tcp"|cut -d: -f2`

            if ! is_numeric $smtp_nbr
            then
                smtp_nbr='25'
            fi
        fi
    else
        # error - set default
        smtp_nbr='25'
    fi

    echo $smtp_nbr
}

#----------------------------------------------------------------------------------------
# generate squirrelmail configuration
# $1 - webmail instance
#----------------------------------------------------------------------------------------
create_squirrelmail_conf ()
{
    wm_nbr=$1
    eval wm_doc_root='$WEBMAIL_'${wm_nbr}'_DOCUMENT_ROOT'
    eval wm_active='$WEBMAIL_'${wm_nbr}'_ACTIVE'

    if [ "$START_WEBMAIL" = "yes" -a "$wm_active" = "yes" ]
    then 
        if [ $wm_nbr -ne 0 -a $wm_nbr -le $WEBMAIL_N ]
        then
            mecho "- creating webmail configuration ..."

            eval wm_orga_name='$WEBMAIL_'${wm_nbr}'_ORGA_NAME'
            eval wm_orga_logo='$WEBMAIL_'${wm_nbr}'_ORGA_LOGO'
            eval wm_orga_logo_width='$WEBMAIL_'${wm_nbr}'_ORGA_LOGO_WIDTH'
            eval wm_orga_logo_height='$WEBMAIL_'${wm_nbr}'_ORGA_LOGO_HEIGHT'
            eval wm_orga_title='$WEBMAIL_'${wm_nbr}'_ORGA_TITLE'
            eval wm_orga_signout_page='$WEBMAIL_'${wm_nbr}'_ORGA_SIGNOUT_PAGE'
            eval wm_orga_top_frame='$WEBMAIL_'${wm_nbr}'_ORGA_TOP_FRAME'
            eval wm_orga_provider_url='$WEBMAIL_'${wm_nbr}'_ORGA_PROVIDER_URL'
            eval wm_orga_provider_name='$WEBMAIL_'${wm_nbr}'_ORGA_PROVIDER_NAME'
            eval wm_orga_def_language='$WEBMAIL_'${wm_nbr}'_ORGA_DEF_LANGUAGE'
            
            eval wm_server_domain='$WEBMAIL_'${wm_nbr}'_SERVER_DOMAIN'
            eval wm_server_domain_check='$WEBMAIL_'${wm_nbr}'_SERVER_DOMAIN_CHECK'
            eval wm_imap_hostport='$WEBMAIL_'${wm_nbr}'_SERVER_IMAP_HOST'
            eval wm_imap_auth='$WEBMAIL_'${wm_nbr}'_SERVER_IMAP_AUTH'
            eval wm_imap_type='$WEBMAIL_'${wm_nbr}'_SERVER_IMAP_TYPE'
            eval wm_imap_transport='$WEBMAIL_'${wm_nbr}'_SERVER_IMAP_TRANSPORT'
            eval wm_smtp_hostport='$WEBMAIL_'${wm_nbr}'_SERVER_SMTP_HOST'
            eval wm_smtp_auth='$WEBMAIL_'${wm_nbr}'_SERVER_SMTP_AUTH'
            eval wm_smtp_transport='$WEBMAIL_'${wm_nbr}'_SERVER_SMTP_TRANSPORT'
            
            eval wm_mv_msgs_to_trash='$WEBMAIL_'${wm_nbr}'_FOLDER_MOVE_MSGS_TO_TRASH'
            eval wm_mv_msgs_to_send='$WEBMAIL_'${wm_nbr}'_FOLDER_MOVE_MSGS_TO_SEND'
            eval wm_mv_msgs_to_draft='$WEBMAIL_'${wm_nbr}'_FOLDER_MOVE_MSGS_TO_DRAFT'
            eval wm_auto_expunge='$WEBMAIL_'${wm_nbr}'_FOLDER_AUTO_EXPUNGE'
            eval wm_mv_folder_to_trash='$WEBMAIL_'${wm_nbr}'_FOLDER_MOVE_FOLDER_TO_TRASH'

            eval wm_folder_spec_list_first='$WEBMAIL_'${wm_nbr}'_FOLDER_SPECIAL_LIST_FIRST'
            eval wm_folder_spec_use_color='$WEBMAIL_'${wm_nbr}'_FOLDER_SPECIAL_USE_COLOR'
            eval wm_folder_unseen_notify='$WEBMAIL_'${wm_nbr}'_FOLDER_UNSEEN_NOTIFY'
            eval wm_folder_unseen_type='$WEBMAIL_'${wm_nbr}'_FOLDER_UNSEEN_TYPE'

            eval wm_general_def_charset='$WEBMAIL_'${wm_nbr}'_GENERAL_DEF_CHARSET'
            eval wm_general_lef_frame_width='$WEBMAIL_'${wm_nbr}'_GENERAL_LEFT_FRAME_WIDTH'
            eval wm_general_allow_prio='$WEBMAIL_'${wm_nbr}'_GENERAL_ALLOW_PRIORITY_USE'
            eval wm_general_allow_receipt='$WEBMAIL_'${wm_nbr}'_GENERAL_ALLOW_RECEIPTS_USE'
            eval wm_general_allow_ident='$WEBMAIL_'${wm_nbr}'_GENERAL_ALLOW_IDENTITY_EDIT'

            eval wm_plugins_use_all='$WEBMAIL_'${wm_nbr}'_PLUGINS_USE_ALL'
            eval wm_plugins_n='$WEBMAIL_'${wm_nbr}'_PLUGINS_N'

            eval wm_themes_def='$WEBMAIL_'${wm_nbr}'_THEMES_DEFAULT'
            eval wm_themes_n='$WEBMAIL_'${wm_nbr}'_THEMES_N'
            eval wm_themes_use_all='$WEBMAIL_'${wm_nbr}'_THEMES_USE_ALL'

            eval wm_globaddr_n='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_N'
            eval wm_globaddr_writeable='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_WRITEABLE'

            eval wm_globldap_n='$WEBMAIL_'${wm_nbr}'_GLOBADDR_LDAP_N'

            {
                echo "<?php"
                echo
                echo "/********************************************************************************"
                echo " * webmail configuration file generated by $pgmname"
                echo " *"
                echo " * Do not edit this file, edit $webmailfile"
                echo " * Creation date: `date`"
                echo " *"

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail
                        echo " * Configuration type: mail"
                        ;;
                    vmail)
                        # vmail
                        echo " * Configuration type: vmail"
                        ;;
                    *)
                        # none local
                        echo " * Configuration type: non-local"
                        ;;
                esac

                echo " *******************************************************************************/"
                echo

                #     /**
                #      * BEFORE EDITING THIS FILE!
                #      *
                #      * don't edit this file directly.  Copy it to config.php before you
                #      * edit it.  However, it is best to use the configuration script
                #      * conf.pl if at all possible.  That is the easiest and cleanest way
                #      * to configure.
                #     */

                #     /* Do not change this value. */
                echo "global \$version;"
                echo "\$config_version       = '1.4.0';"
                echo "\$config_use_color     = 1;"

                #    /**
                #     * (since SM v1.4.8)
                #     * Here you can set the base part of the SquirrelMail URL.
                #     * It is normally autodetected but if that fails, use this
                #     * option to override.
                #     * It should contain only the protocol and hostname/port
                #     * parts of the URL; the full path will be appended
                #     * automatically.
                #     * 
                #     * Examples: http://webmail.example.org
                #     *           http://webmail.example.com:8080
                #     *           https://webmail.example.com:6691
                #     *
                #     * Do not add any path elements.
                #     */
                echo "\$config_location_base = '';"

                echo

                echo "/* organisation settings */"
                #     /* Organization's name */
                #     global $org_name;
                echo "\$org_name        = \"$wm_orga_name\";"

                #    /* Organization's logo picture (blank if none) */
                #    global $org_logo;
                echo "\$org_logo        = SM_PATH . '${wm_orga_logo}';"

                #    /*  The width of the logo (0 for default) */
                #    global $org_logo_width;
                if [ "${wm_orga_logo_width}" = "" ]
                then
                    # set default
                    echo "\$org_logo_width  = '0';"
                else
                    echo "\$org_logo_width  = '${wm_orga_logo_width}';"
                fi

                #    /*  The height of the logo (0 for default) */
                #    global $org_logo_height;
                if [ "${wm_orga_logo_height}" = "" ]
                then
                    # set default
                    echo "\$org_logo_height = '0';"
                else
                    echo "\$org_logo_height = '${wm_orga_logo_height}';"
                fi

                #    /**
                #     * Webmail Title
                #     *   This is the web page title that appears at the top of the browser window.
                #     */
                #    global $org_title;
                echo "\$org_title       = \"$wm_orga_title\";"

                #    /**
                #     * Rather than going to the signout.php page (which only allows you
                #     * to sign back in), setting signout_page allows you to sign the user
                #     * out and then redirect to whatever page you want. For instance,
                #     * the following would return the user to your home page:
                #     *   $signout_page = '/';
                #     * Set to the empty string to continue to use the default signout page.
                #     */
                #    global $signout_page;
                echo "\$signout_page    = '$wm_orga_signout_page';"

                #    /**
                #     * By default SquirrelMail takes up the whole browser window,
                #     * this allows you to embed it within sites using frames. Set
                #     * this to the frame you want it to stay in.
                #     */
                #    global $frame_top;
                echo "\$frame_top       = '$wm_orga_top_frame';"
                echo

                #    /**
                #     * Provider link (since SM v1.3.x)
                #     *   Here you can set the link on the right of the page.
                #     */
                #    global $provider_uri;
                echo "\$provider_uri  = '$wm_orga_provider_url';"

                #    /**
                #     * Provider name (since SM v1.3.x)
                #     *   Here you can set the name of the link on the right of the page.
                #     */
                #    global $provider_name;
                echo "\$provider_name = '$wm_orga_provider_name';"
                echo

                #    /* This is a message that is displayed immediately after a user logs in. */
                #    global $motd;
                echo "\$motd = \"\";"
                echo

                #    /**
                #     * Default language
                #     *   This is the default language. It is used as a last resort
                #     *   if SquirrelMail can't figure out which language to display.
                #     *   Use the two-letter code.
                #     */
                #    global $quirrelmail_default_language;
                echo "\$squirrelmail_default_language = '$wm_orga_def_language';"

                #   /**
                #    * Lossy Encoding Control
                #    *
                #    * This option allows charset conversions when output charset does not support
                #    * all symbols used in original charset. Symbols unsupported by output charset
                #    * will be replaced with question marks.
                #    */
                #    global bool $lossy_encoding
                echo "\$lossy_encoding        = false;"          # (SM v1.4.5)
                echo
                echo "/* server settings */"
                #    /**
                #     * The domain part of local email addresses.
                #     *   This is for all messages sent out from this server.
                #     *   Reply address is generated by $username@$domain
                #     * Example: In bob@foo.com, foo.com is the domain.
                #     */
                #    global $domain;
                echo "\$domain                 = '$wm_server_domain';"

                #    /* The dns name and port for your imap server. */
                #    global $imapServerAddress, $imapPort;

                echo "${wm_imap_hostport}" | grep -q ":"

                if [ $? -eq 0 ]
                then
                    # split hostname and port
                    wm_imap_host="`echo ${wm_imap_hostport} | cut -d: -f1`"
                    wm_imap_port="`echo ${wm_imap_hostport} | cut -d: -f2`"
                else
                    # hostname only
                    wm_imap_host="${wm_imap_hostport}"
                    wm_imap_port='143'
                fi

                if [ "${wm_imap_port}" = "" ]
                then
                    # set default port
                    wm_imap_port="143"
                fi

                echo "\$imapServerAddress      = '${wm_imap_host}';"
                echo "\$imapPort               = ${wm_imap_port};"

                #    /**
                #     * Each IMAP server has its own quirks.  As much as we tried to stick
                #     * to standards, it doesn't help much if the IMAP server doesn't follow
                #     * the same principles.  We have made some work-arounds for some of
                #     * these servers.  If you would like to use them, please select your
                #     * IMAP server.  If you do not wish to use these work-arounds, you can
                #     * set this to "other", and none will be used.
                #     *     bincimap    = Binc IMAP server
                #     *     courier     = Courier IMAP server
                #     *     cyrus       = Cyrus IMAP server
                #     *     dovecot     = Dovecot Secure IMAP server
                #     *     exchange    = Microsoft Exchange IMAP server
                #     *     hmailserver = hMailServer
                #     *     macosx      = Mac OS X Mailserver
                #     *     mercury32   = Mercury/32
                #     *     uw          = University of Washington's IMAP server
                #     *     other       = Not one of the above servers
                #     */
                #    global $imap_server_type;

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail (uw)
                        echo "\$imap_server_type       = 'uw';"

                        if [ "${wm_imap_type}" != 'uw' ]
                        then
                            eval "WEBMAIL_${wm_nbr}_SERVER_IMAP_TYPE='uw'"
                            write_to_config_log -warn "Parameter WEBMAIL_${wm_nbr}_SERVER_IMAP_TYPE='${wm_imap_type}' has been set although"
                            write_to_config_log -warn -ff "it should be set to 'uw'. Please check the configuration!"
                        fi
                        ;;
                    vmail)
                        # vmail
                        # set platform specific parameters
                        case ${EISFAIR_SYSTEM} in
                            eisfair-2)
                                # eisfair-2 (dovecot)
                                echo "\$imap_server_type       = 'dovecot';"

                                if [ "${wm_imap_type}" != 'dovecot' ]
                                then
                                    eval "WEBMAIL_${wm_nbr}_SERVER_IMAP_TYPE='dovecot'"
                                    write_to_config_log -warn "Parameter WEBMAIL_${wm_nbr}_SERVER_IMAP_TYPE='${wm_imap_type}' has been set although"
                                    write_to_config_log -warn -ff "it should be set to 'dovecot'. Please check the configuration!"
                                fi
                                ;;
                            *)
                                # default to eisfair-1 (courier)
                                echo "\$imap_server_type       = 'courier';"

                                if [ "${wm_imap_type}" != 'courier' ]
                                then
                                    eval "WEBMAIL_${wm_nbr}_SERVER_IMAP_TYPE='courier'"
                                    write_to_config_log -warn "Parameter WEBMAIL_${wm_nbr}_SERVER_IMAP_TYPE='${wm_imap_type}' has been set although"
                                    write_to_config_log -warn -ff "it should be set to 'courier'. Please check the configuration!"
                                fi
                                ;;
                        esac
                        ;;
                    *)
                        # none local
                        echo "\$imap_server_type       = '${wm_imap_type}';"
                        ;;
                esac

                #    /**
                #     * (since SM v1.3.x)
                #     * What authentication mechanism do you want to use for IMAP connections?
                #     * plain      - Plaintext. If you can do better, you probably should.
                #     * cram-md5   - Slightly better than plaintext.
                #     * digest-md5 - Privacy protection - better than cram-md5.
                #     *
                #     * eisfair-UW imap server supports:
                #     * LOGIN:           SUPPORTED
                #     * CRAM-MD5:        SUPPORTED
                #     * DIGEST-MD5:      NOT SUPPORTED
                #     */
                #    global $imap_auth_mech;

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail
                        if [ "${MAIL_USER_USE_MAILONLY_PASSWORDS}" = "yes" ]
                        then
                            # different passwords are enabled
                            echo "\$imap_auth_mech         = 'cram-md5';"
                        else
                            echo "\$imap_auth_mech         = 'login';"
                        fi
                        ;;
                    vmail)
                        # vmail
                        echo "\$imap_auth_mech         = 'login';"
                        ;;
                    *)
                        # none local
                        case ${wm_imap_auth} in
                            digest)
                                echo "\$imap_auth_mech         = 'digest-md5';"
                                ;;
                            md5)
                                echo "\$imap_auth_mech         = 'cram-md5';"
                                ;;
                            login)
                                echo "\$imap_auth_mech         = '${wm_imap_auth}';"
                                ;;
                        esac
                        ;;
                esac

                #    /**
                #     * (since SM v1.3.x)
                #     * TLS (Transport Layer Security) encrypts the traffic between server
                #     * and client. If you're familiar with SSL, you get the idea. To use
                #     * this feature, your SMTP server must offer TLS capability, plus
                #     * PHP 4.3.x with OpenSSL support.
                #     * If your SMTP server is localhost, you can safely disable this.
                #     * If it is remote, you may wish to seriously consider enabling this.
                #     */
                #    global $use_imap_tls;

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail
                        if [ "${wm_imap_transport}" = "tls" ]
                        then
                            echo "\$use_imap_tls           = true;"

                            # check if secure port has been set
                            if [ "${wm_imap_port}" != "993" ]
                            then
                                write_to_config_log -warn "Parameter POP3IMAP_TRANSPORT='tls' has been set therefore it is"
                                write_to_config_log -warn -ff "necessary to set parameter WEBMAIL_${wm_nbr}_SERVER_IMAP_HOST='${wm_imap_host}:993' too."
                            fi
                        else
                            # default
                            echo "\$use_imap_tls           = false;"

                            if [ "${POP3IMAP_TRANSPORT}" = "tls" ]
                            then
                                write_to_config_log -info "Parameter POP3IMAP_TRANSPORT='tls' has been set therefore it is"
                                write_to_config_log -info -ff "recommended to set parameter WEBMAIL_${wm_nbr}_SERVER_IMAP_TRANSPORT='tls' too."
                            fi
                        fi
                        ;;
                    vmail)
                        # vmail
                        echo "\$use_imap_tls           = false;"
                        ;;
                    *)
                        # none local
                        if [ "${wm_imap_transport}" = "tls" ]
                        then
                            echo "\$use_imap_tls           = true;"
                        else
                            # default
                            echo "\$use_imap_tls           = false;"
                        fi
                        ;;
                esac

                #    /* Your SMTP server and port number (usually the same as the IMAP server). */
                #    global $smtpServerAddress, $smtpPort;

                echo "${wm_smtp_hostport}" | grep -q ":"

                if [ $? -eq 0 ]
                then
                    # split hostname and port
                    wm_smtp_host="`echo ${wm_smtp_hostport} | cut -d: -f1`"
                    wm_smtp_port="`echo ${wm_smtp_hostport} | cut -d: -f2`"
                else
                    # hostname only
                    wm_smtp_host="${wm_smtp_hostport}"
                    wm_smtp_port='25'
                fi

                if [ "${wm_smtp_port}" = "" ]
                then
                    # set default value
                    wm_smtp_port='25'
                fi

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail - read port from mail package
                        if [ "${SMTP_SERVER_TRANSPORT}" = "tls" -a "${SMTP_SERVER_SSMTP}" = "yes" ]
                        then
                            # tls connection requested
                            mail_smtp_port=`get_smtp_port ${SMTP_SERVER_SSMTP_LISTEN_PORT}`

                            if [ "${mail_smtp_port}" = "" ]
                            then
                                mail_smtp_port='465'
                            fi

                            if [ "${wm_smtp_port}" != "${mail_smtp_port}" ]
                            then
                                mecho -info "Make sure that the SSMTP listen port is properly set in both mail"
                                mecho -info "and webmail package!"
                                write_to_config_log -info "Make sure that the SSMTP listen port is properly set in both mail"
                                write_to_config_log -info -ff "and webmail package!"
                            fi
                        else
                            # normal connection
                            mail_smtp_port=`get_smtp_port ${SMTP_LISTEN_PORT}`

                            if [ "${mail_smtp_port}" = "" ]
                            then
                                mail_smtp_port='25'
                            fi

                            if [ "${wm_smtp_port}" != "${mail_smtp_port}" ]
                            then
                                mecho -info "Make sure that the SMTP listen port is properly set in both mail"
                                mecho -info "and webmail package!"
                                write_to_config_log -info "Make sure that the SMTP listen port is properly set in both mail"
                                write_to_config_log -info -ff "and webmail package!"
                            fi
                        fi

                        # check hostname
                        if [ "${wm_smtp_host}" != "localhost" -a "${wm_smtp_host}" != "127.0.0.1" ]
                        then
                            write_to_config_log -warn "Parameter WEBMAIL_${wm_nbr}_SERVER_SMTP_HOST='localhost' has not been set although a local mail"
                            write_to_config_log -warn -ff "package has been installed!"
                        fi
                        ;;
                    vmail)
                        # vmail - use default port
                        # check hostname
                        if [ "${wm_smtp_host}" != "localhost" -a "${wm_smtp_host}" != "127.0.0.1" ]
                        then
                            write_to_config_log -warn "Parameter WEBMAIL_${wm_nbr}_SERVER_SMTP_HOST='localhost' has not been set although a local vmail"
                            write_to_config_log -warn -ff "package has been installed!"
                        fi
                        ;;
                    *)
                        # none local
                        # check port number
                        if [ "${wm_smtp_port}" != "25" ]
                        then
                            write_to_config_log -warn "Parameter WEBMAIL_${wm_nbr}_SERVER_SMTP_HOST='...:${wm_smtp_port}' has been set to a non-standard port!"
                            write_to_config_log -warn "This might cause a communication problem!"
                        fi

                        # check hostname
                        if [ "${wm_smtp_host}" = "localhost" -o "${wm_smtp_host}" = "127.0.0.1" ]
                        then
                            write_to_config_log -error "Parameter WEBMAIL_${wm_nbr}_SERVER_SMTP_HOST='localhost' has been set although no local mail or"
                            write_to_config_log -error -ff "vmail package has been installed!"
                        fi
                        ;;
                esac

                echo "\$smtpServerAddress      = '${wm_smtp_host}';"
                echo "\$smtpPort               = ${wm_smtp_port};"

                #    /**
                #     * (since SM v1.3.x)
                #     * What authentication mechanism do you want to use for SMTP connections?
                #     * none       - Your SMTP server does not require authorization.
                #     * plain      - Plaintext. If you can do better, you probably should.
                #     * cram-md5   - Slightly better than plaintext.
                #     * digest-md5 - Privacy protection - better than cram-md5.
                #     *
                #     * eisfair-UW imap server supports:
                #     * none:           SUPPORTED
                #     * plain:          SUPPORTED
                #     * CRAM-MD5:       SUPPORTED
                #     * DIGEST-MD5:     SUPPORTED
                #     */
                #    global $smtp_auth_mech;

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail
                        case ${SMTP_AUTH_TYPE} in
                            none)
                                # no authentication
                                echo "\$smtp_auth_mech         = 'none';"
                                echo "\$smtp_sitewide_user     = '';"                               # (SM v1.4.11)
                                echo "\$smtp_sitewide_pass     = '';"                               # (SM v1.4.11)
                                ;;
                            user|user_light)
                                # SM 1.2.10 supports only RFC2554
                              # echo "\$use_authenticated_smtp = true;"

                                # server authentication
                                if [ "${MAIL_USER_USE_MAILONLY_PASSWORDS}" = "yes" ]
                                then
                                    # cram-md5 is only advertised if this parameter has
                                    # been set: MAIL_USER_USE_MAILONLY_PASSWORDS="yes"
                                    echo "\$smtp_auth_mech         = 'cram-md5';"
                                else
                                    # use alternate authentication method
                                    echo "\$smtp_auth_mech         = 'login';"
                                fi
                                echo "\$smtp_sitewide_user     = '';"                               # (SM v1.4.11)
                                echo "\$smtp_sitewide_pass     = '';"                               # (SM v1.4.11)
                                ;;
                            server|server_light)
                                if [ "${MAIL_USER_USE_MAILONLY_PASSWORDS}" = "yes" ]
                                then
                                    # cram-md5 is only advertised if this parameter has
                                    # been set: MAIL_USER_USE_MAILONLY_PASSWORDS="yes"
                                    echo "\$smtp_auth_mech         = 'cram-md5';"
                                else
                                    # use alternate authentication method
                                    echo "\$smtp_auth_mech         = 'login';"
                                fi
                                #    /* SMTP authentication uses IMAP username and password by default. */
                                echo "\$smtp_sitewide_user     = '${SMTP_AUTH_USER}';"              # (SM v1.4.11)
                                echo "\$smtp_sitewide_pass     = '${SMTP_AUTH_PASS}';"              # (SM v1.4.11)
                                ;;
                        esac
                        ;;
                    vmail)
                        # vmail - no authentication
                        echo "\$smtp_auth_mech         = 'login';"
                        ;;
                    *)
                        # none local
                        case ${wm_smtp_auth} in
                            digest)
                                echo "\$smtp_auth_mech         = 'digest-md5';"
                                ;;
                            md5)
                                echo "\$smtp_auth_mech         = 'cram-md5';"
                                ;;
                            login|none)
                                echo "\$smtp_auth_mech         = '${wm_smtp_auth}';"
                                ;;
                        esac
                        ;;
                esac

                #    /**
                #     * (since SM v1.3.x)
                #     * TLS (Transport Layer Security) encrypts the traffic between server
                #     * and client. If you're familiar with SSL, you get the idea. To use
                #     * this feature, your SMTP server must offer TLS capability, plus
                #     * PHP 4.3.x with OpenSSL support.
                #     * If your SMTP server is localhost, you can safely disable this.
                #     * If it is remote, you may wish to seriously consider enabling this.
                #     */
                #    global $use_smtp_tls;

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail
                        if [ "${wm_smtp_transport}" = "tls" ]
                        then
                            echo "\$use_smtp_tls           = true;"

                            if [ "${SMTP_SERVER_TRANSPORT}" = "tls" -a "${SMTP_SERVER_SSMTP}" = "yes" ]
                            then

                                # check if secure port has been set
                                if [ "${wm_smtp_port}" != "465" ]
                                then
                                    write_to_config_log -warn "Parameters SMTP_SERVER_TRANSPORT='tls' and SMTP_SERVER_SSMTP='yes'"
                                    write_to_config_log -warn -ff "have been set therefore it's recommended to set parameter"
                                    write_to_config_log -warn -ff "WEBMAIL_${wm_nbr}_SERVER_SMTP_HOST='${wm_smtp_host}:465' too."
                                fi
                            else
                                # unsupported parameter combination
                                echo "\$use_smtp_tls           = false;"

                                mecho -error "If you want to enable TLS support you will need to set"
                                mecho -error "at least SMTP_SERVER_SSMTP='yes'."
                                write_to_config_log -error "If you want to enable TLS support you will need to set"
                                write_to_config_log -error -ff "at least SMTP_SERVER_SSMTP='yes'."
                            fi
                        else
                            echo "\$use_smtp_tls           = false;"
                        fi
                        ;;
                    vmail)
                        # vmail
                        echo "\$use_smtp_tls           = false;"
                        ;;
                    *)
                        # none local
                        if [ "${wm_smtp_transport}" = "tls" ]
                        then
                            echo "\$use_smtp_tls           = true;"
                        else
                            # default
                            echo "\$use_smtp_tls           = false;"
                        fi
                        ;;
                esac

                #    /**
                #     * Enable this if you want to use POP3 before SMTP.
                #     * Your server must support this in order for SquirrelMail to work with it.
                #     */
                echo "\$pop_before_smtp        = false;"


                #    /**
                #     * (since SM v1.4.16)
                #     * If the address of the POP server is not the same as your SMTP server,
                #     * you may specify it here. Leave blank (to clear this, enter only spaces)
                #     * to use the same address as your SMTP server.
                #     */
                echo "\$pop_before_smtp_host   = '';"

                #    /**
                #     * Uncomment this if you want to deliver locally using sendmail
                #     * instead of connecting to a SMTP-server.
                #     */
                #    #global $useSendmail, $sendmail_path;

                # generally used for mail and vmail
                echo "\$useSendmail            = false;"
                echo "\$sendmail_path          = '/usr/sbin/sendmail';"
                echo "\$sendmail_args          = '-i -t';"                          # (SM v1.4.9)

                #    /**
                #     * If you are running on a machine that doesn't have the tm_gmtoff
                #     * value in your time structure and if you are in a time zone that
                #     * has a negative offset, you need to set this value to 1. This is
                #     * typically people in the US that are running Solaris 7.
                #     */
                #    global $invert_time;
                echo "\$invert_time            = false;"

                #    /**
                #     * This is the delimiter that your IMAP server uses to distinguish between
                #     * folders. For example, Cyrus uses '.' as the delimiter and a complete
                #     * folder would look like 'INBOX.Friends.Bob', while UW uses '/' and would
                #     * look like 'INBOX/Friends/Bob'.  Normally this should be left at 'detect'
                #     * but if you are sure you know what delimiter your server uses, you can
                #     * specify it here.
                #     *
                #     * To have it autodetect the delimiter, set it to 'detect'.
                #     */

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail (uw)
                        echo "\$optional_delimiter     = '/';"
                        ;;
                    vmail)
                        # vmail
                        case ${EISFAIR_SYSTEM} in
                            eisfair-2)
                                # eisfair-2 (dovecot)
                                echo "\$optional_delimiter     = 'detect';"
                                ;;
                            *)
                                # default to eisfair-1 (courier)
                                echo "\$optional_delimiter     = '.';"
                                ;;
                        esac
                        ;;
                    *)
                        # none local
                        case ${wm_imap_type} in
                            courier)
                                echo "\$optional_delimiter     = '.';"
                                ;;
                            dovecot)
                                echo "\$optional_delimiter     = 'detect';"
                                ;;
                            uw)
                                echo "\$optional_delimiter     = '/';"
                                ;;
                        esac
                        ;;
                esac

                #    /**
                #     * SquirrelMail header encryption
                #     *
                #     * Encryption key allows to hide SquirrelMail Received: headers
                #     * in outbound messages. Interface uses encryption key to encode
                #     * username, remote address and proxied address, then stores encoded
                #     * information in X-Squirrel-* headers.
                #     *
                #     * Warning: used encryption function is not bulletproof. When used
                #     * with static encryption keys, it provides only minimal security
                #     * measures and information can be decoded quickly.
                #     *
                #     * Encoded information can be decoded with decrypt_headers.php script
                #     * from SquirrelMail contrib/ directory.
                #     */
                #     global string $encode_header_key
                echo "\$encode_header_key      = '';"       # (SM v1.4.5)

                echo
                echo "/* folder settings */"
                #    /**
                #     * Many servers store mail in your home directory. With this, they
                #     * store them in a subdirectory: mail/ or Mail/, etc. If your server
                #     * does this, please set this to what the default mail folder should
                #     * be. This is still a user preference, so they can change it if it
                #     * is different for each user.
                #     *
                #     * Example:
                #     *     $default_folder_prefix = 'mail/';
                #     *        -- or --
                #     *     $default_folder_prefix = 'Mail/folders/';
                #     *
                #     * If you do not use this, set it to the empty string.
                #     */
                #    global $default_folder_prefix;

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail (uw)
                        echo "\$default_folder_prefix          = '';"
                        echo "\$disp_default_folder_prefix     = '';"
                        ;;
                    vmail)
                        # vmail
                        case ${EISFAIR_SYSTEM} in
                            eisfair-2)
                                # eisfair-2 (dovecot)
                                echo "\$default_folder_prefix          = '';"
                                echo "\$disp_default_folder_prefix     = '<none>';"
                                ;;
                            *)
                                # default to eisfair-1 (courier)
                                echo "\$default_folder_prefix          = 'INBOX.';"
                                echo "\$disp_default_folder_prefix     = 'INBOX.';"
                                ;;
                        esac
                        ;;
                    *)
                        # none local
                        case ${wm_imap_type} in
                            courier)
                                echo "\$default_folder_prefix          = 'INBOX.';"
                                echo "\$disp_default_folder_prefix     = 'INBOX.';"
                                ;;
                            dovecot)
                                echo "\$default_folder_prefix          = '';"
                                echo "\$disp_default_folder_prefix     = '<none>';"
                                ;;
                            uw)
                                echo "\$default_folder_prefix          = '';"
                                echo "\$disp_default_folder_prefix     = '';"
                                ;;
                        esac
                        ;;
                esac

                #    /**
                #     * The following are related to deleting messages.
                #     *   $move_to_trash
                #     *      if this is set to 'true', when 'delete' is pressed, it
                #     *      will attempt to move the selected messages to the folder
                #     *      named $trash_folder. If it's set to 'false', we won't even
                #     *      attempt to move the messages, just delete them.
                #     *   $trash_folder
                #     *      This is the path to the default trash folder. For Cyrus
                #     *      IMAP, it would be 'INBOX.Trash', but for UW it would be
                #     *      'Trash'. We need the full path name here.
                #     *   $auto_expunge
                #     *      If this is true, when a message is moved or copied, the
                #     *      source mailbox will get expunged, removing all messages
                #     *      marked 'Deleted'.
                #     *   $sent_folder
                #     *      This is the path to where Sent messages will be stored.
                #     *   $delete_folder
                #     *      If this is true, when a folder is deleted then it will
                #     *      not get moved into the Trash folder.
                #     */
                #    global $default_move_to_trash, $default_move_to_sent, $default_save_as_draft;
                #    global $trash_folder, $sent_folder, $draft_folder, $auto_expunge;
                #    global $delete_folder;
                echo "\$trash_folder                   = 'Trash';"
                echo "\$sent_folder                    = 'Sent';"
                echo "\$draft_folder                   = 'Drafts';"

                if [ "$wm_mv_msgs_to_trash" = "yes" ]
                then
                    echo "\$default_move_to_trash          = true;"
                else
                    echo "\$default_move_to_trash          = false;"
                fi

                if [ "$wm_mv_msgs_to_send" = "yes" ]
                then
                    echo "\$default_move_to_sent           = true;"
                else
                    echo "\$default_move_to_sent           = false;"
                fi

                if [ "$wm_mv_msgs_to_draft" = "yes" ]
                then
                    echo "\$default_save_as_draft          = true;"
                else
                    echo "\$default_save_as_draft          = false;"
                fi

                if [ "$wm_auto_expunge" = "yes" ]
                then
                    echo "\$auto_expunge                   = true;"
                else
                    echo "\$auto_expunge                   = false;"
                fi

                if [ "$wm_mv_folder_to_trash" = "yes" ]
                then
                    echo "\$delete_folder                  = false;"
                else
                    echo "\$delete_folder                  = true;"
                fi

                #    /**
                #     * If you do not wish to give them the option to change this, set it
                #     * to false. Otherwise, if it is true, they can change the folder prefix
                #     * to be anything.
                #     */
                #    global $show_prefix_option;

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail (uw)
                        echo "\$show_prefix_option             = true;"
                        ;;
                    vmail)
                        # vmail
                        case ${EISFAIR_SYSTEM} in
                            eisfair-2)
                                # eisfair-2 (dovecot)
                                echo "\$show_prefix_option             = false;"
                                ;;
                            *)
                                # default to eisfair-1 (courier)
                                echo "\$show_prefix_option             = false;"
                                ;;
                        esac
                        ;;
                    *)
                        # none local
                        case ${wm_imap_type} in
                            courier)
                                echo "\$show_prefix_option             = false;"
                                ;;
                            dovecot)
                                echo "\$show_prefix_option             = false;"
                                ;;
                            uw)
                                echo "\$show_prefix_option             = true;"
                                ;;
                        esac
                        ;;
                esac

                #    /* Whether or not to list the special folders first (true/false). */
                #    global $list_special_folders_first;
                if [ "$wm_folder_spec_list_first" = "yes" ]
                then
                    echo "\$list_special_folders_first     = true;"
                else
                    echo "\$list_special_folders_first     = false;"
                fi

                #    /**
                #     * Whether or not to use a special color for special folders. If not,
                #     * special folders will be the same color as the other folders.
                #     */
                #    global $use_special_folder_color;
                if [ "$wm_folder_spec_use_color" = "yes" ]
                then
                    echo "\$use_special_folder_color       = true;"
                else
                    echo "\$use_special_folder_color       = false;"
                fi

                #    /**
                #    * Are all your folders subfolders of INBOX (i.e. cyrus IMAP server).
                #    * If you are unsure, set it to false.
                #    */
                #    global $default_sub_of_inbox;
                echo "\$default_sub_of_inbox           = false;"

                #    /**
                #     * Some IMAP daemons (UW) handle folders weird. They only allow a
                #     * folder to contain either messages or other folders, not both at
                #     * the same time. This option controls whether or not to display an
                #     * option during folder creation. The option toggles which type of
                #     * folder it should be.
                #     *
                #     * If this option confuses you, just set it to 'true'. You can not hurt
                #     * anything if it's true, but some servers will respond weird if it's
                #     * false. (Cyrus works fine whether it's true OR false).
                #     */
                #    global $show_contain_subfolders_option;

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail (uw)
                        echo "\$show_contain_subfolders_option = true;"
                        ;;
                    vmail)
                        # vmail
                        case ${EISFAIR_SYSTEM} in
                            eisfair-2)
                                # eisfair-2 (dovecot)
                                echo "\$show_contain_subfolders_option = false;"
                                ;;
                            *)
                                # default to eisfair-1 (courier)
                                echo "\$show_contain_subfolders_option = false;"
                                ;;
                        esac
                        ;;
                    *)
                        # none local
                        case ${wm_imap_type} in
                            courier)
                                echo "\$show_contain_subfolders_option = false;"
                                ;;
                            dovecot)
                                echo "\$show_contain_subfolders_option = false;"
                                ;;
                            uw)
                                echo "\$show_contain_subfolders_option = true;"
                                ;;
                        esac
                        ;;
                esac

                #    /**
                #     * These next two options set the defaults for the way that the
                #     * users see their folder list.
                #     *   $default_unseen_notify
                #     *       Specifies whether or not the users will see the number of
                #     *       unseen in each folder by default and also which folders to
                #     *       do this to. Valid values are: 1=none, 2=inbox, 3=all.
                #     *   $default_unseen_type
                #     *       Specifies the type of notification to give the users by
                #     *       default. Valid choice are: 1=(4), 2=(4,25).
                #     */
                #    global $default_unseen_notify, $default_unseen_type;
                case "$wm_folder_unseen_notify"
                in
                    none)
                        echo "\$default_unseen_notify          = 1;"
                        ;;
                    inbox)
                        echo "\$default_unseen_notify          = 2;"
                        ;;
                    all)
                        echo "\$default_unseen_notify          = 3;"
                        ;;
                esac

                if [ "$wm_folder_unseen_type" = "both" ]
                then
                    echo "\$default_unseen_type            = 2;"
                else
                    echo "\$default_unseen_type            = 1;"
                fi

                #    /**
                #     * Should I create the Sent and Trash folders automatically for
                #     * a new user that doesn't already have them created?
                #     */
                #    global $auto_create_special;
                echo "\$auto_create_special            = true;"

                #    /**
                #     * This enables the no select fix for Cyrus when subfolders
                #     * exist but parent folders do not
                #     */
                #    global $noselect_fix_enable;
                echo "\$noselect_fix_enable            = false;"

                #    /**
                #     * this disables listing all of the folders on the IMAP Server to
                #     * generate the folder subscribe listbox (this can take a long time
                #     * when you have a lot of folders).  Instead, a textbox will be
                #     * displayed allowing users to enter a specific folder name to subscribe to */
                #    global $no_list_for_subscribe;
                echo "\$no_list_for_subscribe          = false;"
                echo

                echo "/* general settings */"
                #    /**
                #     * This option controls what character set is used when sending mail
                #     * and when sending HTMl to the browser. Do not set this to US-ASCII,
                #     * use ISO-8859-1 instead. For cyrillic it is best to use KOI8-R,
                #     * since this implementation is faster than the alternatives.
                #     */
                #    global $default_charset;
                echo "\$default_charset                = '$wm_general_def_charset';"

                #     /**
                #      * Path to the data/ directory
                #      *   It is a possible security hole to have a writable directory
                #      *   under the web server's root directory (ex: /home/httpd/html).
                #      *   For this reason, it is possible to put the data directory
                #      *   anywhere you would like. The path name can be absolute or
                #      *   relative (to the config directory). It doesn't matter. Here
                #      *   are two examples:
                #      *
                #      * Absolute:
                #      *   $data_dir = '/usr/local/squirrelmail/data/';
                #      *
                #      * Relative (to the config directory):
                #      *   $data_dir = '../data/';
                #      */
                #     global $data_dir;
                echo "\$data_dir                       = '${webmail_data_path}/';"

                #    /**
                #     * Path to directory used for storing attachments while a mail is
                #     * being sent. There are a few security considerations regarding
                #     * this directory:
                #     *    + It should have the permission 733 (rwx-wx-wx) to make it
                #     *      impossible for a random person with access to the webserver to
                #     *      list files in this directory. Confidential data might be laying
                #     *      around there.
                #     *    + Since the webserver is not able to list the files in the content
                #     *       is also impossible for the webserver to delete files lying around
                #     *       there for too long.
                #     *    + It should probably be another directory than data_dir.
                #     */
                #    global $attachment_dir;
                echo "\$attachment_dir                 = '${webmail_attach_path}/';"

                #    /* Hash level used for data directory. */
                #    global $dir_hash_level;
                echo "\$dir_hash_level                 = 4;"

                #    /**
                #     * This is the default size of the folder list. Default
                #     * is 150, but you can set it to whatever you wish.
                #     */
                #    global $default_left_size;
                echo "\$default_left_size              = '$wm_general_lef_frame_width';"

                #    /**
                #     * Some IMAP servers allow a username (like 'bob') to log in if they use
                #     * uppercase in their name (like 'Bob' or 'BOB'). This creates extra
                #     * preference files.  Toggling this option to true will transparently
                #     * change all usernames to lowercase.
                #     */
                #    global $force_username_lowercase;

                case ${MAIL_INSTALLED} in
                    mail)
                        # mail (uw)
                        echo "\$force_username_lowercase       = false;"
                        ;;
                    vmail)
                        # vmail
                        case ${EISFAIR_SYSTEM} in
                            eisfair-2)
                                # eisfair-2 (dovecot)
                                echo "\$force_username_lowercase       = true;"
                                ;;
                            *)
                                # default to eisfair-1 (courier)
                                echo "\$force_username_lowercase       = false;"
                                ;;
                        esac
                        ;;
                    *)
                        # none local
                        case ${wm_imap_type} in
                            courier)
                                echo "\$force_username_lowercase       = false;"
                                ;;
                            dovecot)
                                echo "\$force_username_lowercase       = true;"
                                ;;
                            uw)
                                echo "\$force_username_lowercase       = false;"
                                ;;
                        esac
                esac

                #    /* Allow users to specify priority of outgoing mail */
                if [ "$wm_general_allow_prio" = "yes" ]
                then
                    echo "\$default_use_priority           = true;"
                else
                    echo "\$default_use_priority           = false;"
                fi

                echo "\$hide_sm_attributions           = false;"

                if [ "$wm_general_allow_receipt" = "yes" ]
                then
                    echo "\$default_use_mdn                = true;"
                else
                    echo "\$default_use_mdn                = false;"
                fi

                #    /**
                #     * If you don't want to allow users to change their email address
                #     * then you can set $edit_identity to false, if you want them to
                #     * not be able to change their full name too then set $edit_name
                #     * to false as well. $edit_name has no effect unless $edit_identity
                #     * is false;
                #     */
                #    global $edit_identity, $edit_name;
                if [ "$wm_general_allow_ident" = "yes" ]
                then
                    echo "\$edit_identity                  = true;"
                else
                    echo "\$edit_identity                  = false;"
                fi

                echo "\$edit_name                      = true;"

                #    /**
                #     * SquirrelMail adds username information to every sent email.
                #     * It is done in order to prevent possible sender forging when
                #     * end users are allowed to change their email and name
                #     * information.
                #     *
                #     * You can disable this header, if you think that it violates
                #     * user's privacy or security. Please note, that setting will
                #     * work only when users are not allowed to change their identity.
                #     *
                #     * See SquirrelMail bug tracker #847107 for more details about it.
                #     * global bool $hide_auth_header
                #     */
                echo "\$hide_auth_header               = false;"    # (SM v1.4.5)

                #    /**
                #     * If you want to enable server side thread sorting options
                #     * Your IMAP server must support the THREAD extension for
                #     * this to work.
                #     */
                #     global $allow_thread_sort;
                echo "\$allow_thread_sort              = true;"

                #    /**
                #     * to use server-side sorting instead of SM client side.
                #     * Your IMAP server must support the SORT extension for this
                #     * to work.
                #     */
                #    global $allow_server_sort;
                echo "\$allow_server_sort              = false;"

                #    /**
                #     * This option allows you to choose if SM uses charset search
                #     * Your IMAP server must support the SEARCH CHARSET command for this to work
                #     */
                echo "\$allow_charset_search           = true;"

                #    /**
                #     * (since SM v1.3.x)
                #     * This option allows you to enable unique identifier (UID) support.
                #     */
                echo "\$uid_support                    = true;"

                #    /**
                #     * (since SM v1.4.16)
                #     * This option allows you to specify that if a user session is initiated
                #     * under a secure (HTTPS, SSL-encrypted) connection, the cookies given to
                #     * the browser will ONLY be transmitted via a secure connection henceforth.
                #     *
                #     * Generally this is a Good Thing, and should NOT be disabled!
                #     */
                echo "\$only_secure_cookies            = true;"
                echo

                #    /**
                #     * To install plugins, just add elements to this array that have
                #     * the plugin directory name relative to the /plugins/ directory.
                #     * For instance, for the 'sqclock' plugin, you'd put a line like
                #     * the following.
                #     *    $plugins[0] = 'sqclock';
                #     *    $plugins[1] = 'attachment_common';
                #     */
                #    global $plugins;

                if [ "$wm_plugins_use_all" = "yes" ]
                then
                    # process all plugins in directory
                    echo "/* list of all plugins */"

                    process_all_plugins ${wm_nbr}
                else
                    if [ $wm_plugins_n -gt 0 ]
                    then
                        # process individual plugins
                        echo "/* list of individual plugins */"

                        process_individual_plugins ${wm_nbr}
                    else
                        # plugins disabled
                        echo "/* use of plugins has been disabled */"
                    fi
                fi

                echo

                #    /**
                #     * Themes
                #     *   You can define your own theme and put it in this directory.
                #     *   You must call it as the example below. You can name the theme
                #     *   whatever you want. For an example of a theme, see the ones
                #     *   included in the config directory.
                #     *
                #     * To add a new theme to the options that users can choose from, just
                #     * add a new number to the array at the bottom, and follow the pattern.
                #     *
                #     * The first one HAS to be here, and is your system's default
                #     * theme. It can be any theme you want.
                #     */
                #    global $theme;

                #    /**
                #     * You may specify a cascading style-sheet (CSS) file to be included
                #     * on each html page generated by SquirrelMail. The CSS file is useful
                #     * for specifying a site-wide font. If you're not familiar with CSS
                #     * files, leave this blank.
                #     */
                echo "\$theme_css        = '';"

                #    /**
                #     * number of default theme
                #     */
                if [ $wm_themes_def -gt $wm_themes_n -o $wm_themes_def -lt 0 ]
                then
                    echo "\$theme_default    = 0;"
                else
                    echo "\$theme_default    = $wm_themes_def;"
                fi

                if [ "$wm_themes_use_all" = "yes" ]
                then
                    # process all themes in directory
                    echo "/* list of all themes */"

                    process_all_themes ${wm_nbr}
                else
                    if [ $wm_themes_n -eq 0 ]
                    then
                        # print default theme
                        echo "/* only default theme will be used */"

                        echo "\$theme[0]['PATH'] = SM_PATH . 'themes/default_theme.php';"
                        echo "\$theme[0]['NAME'] = 'Default Theme';"

                        write_to_config_log -warn "Parameter \"\$WEBMAIL_${wm_nbr}_THEMES_N\" has been set to \"0\", only default theme will be used!"
                    else
                        # process individual themes
                        echo "/* list of individual themes */"

                        process_individual_themes ${wm_nbr}
                    fi
                fi

                echo

                echo "/* database settings */"
                #    /**
                #     * Users may search their addressbook via either a plain HTML or Javascript
                #     * enhanced user interface. This option allows you to set the default choice.
                #     * Set this default choice as either:
                #     *    true  = javascript
                #     *    false = html
                #     */
                #    global $default_use_javascript_addr_book;
                echo "\$default_use_javascript_addr_book = false;"

                #    /**
                #     * If you want to use global file address book, then you
                #     * must set this option to a valid value. If option does
                #     * not have path elements, system assumes that file is
                #     * stored in data directory. If relative path is set, it
                #     * is relative to main squirrelmail directory. If value
                #     * is empty, address book is not enabled.
                #     */
                if [ ${wm_globaddr_n} -ne 0 ]
                then
                    echo "\$abook_global_file                = '${generate_globaddr}-${wm_nbr}';"   # (SM v1.4.5)
                else
                    # write an empty entry to disable the global address book
                    echo "\$abook_global_file                = ' ';"
                fi

                #    /**
                #     * This setting controls writing into global file address
                #     * book options. Address book file must be writeable by
                #     * webserver's user, if you want to enable this option.
                #     */
                if [ "$wm_globaddr_writeable" = "yes" ]
                then
                    echo "\$abook_global_file_writeable      = true;"               # (SM v1.4.5)
                else
                    echo "\$abook_global_file_writeable      = false;"              # (SM v1.4.5)
                fi

                #    /**
                #     * Allow listing of global file address book
                #     */
                echo "\$abook_global_file_listing        = true;"                   # (SM v1.4.9)

                #    /**
                #     * This setting controls space allocated to file based address book
                #     * records. End users will be unable to save address book entry, if
                #     * total entry size (quoted address book fields + 4 delimiters +
                #     * linefeed) exceeds allowed address book length size.
                #     *
                #     * Same setting is applied to personal and global file based address books.
                #     *
                #     * It is strongly recommended to keep default setting value. Change
                #     * it only if you really want to store address book entries that are
                #     * bigger than two kilobytes (2048).
                echo "\$abook_file_line_length           = 2048;"                   # (SM v1.4.9)

                #    /**
                #     * Database-driven private addressbooks
                #     *   DSN (Data Source Name) for a database where the private
                #     *   addressbooks are stored.  See doc/db-backend.txt for more info.
                #     *   If it is not set, the addressbooks are stored in files
                #     *   in the data dir.
                #     *   The DSN is in the format: mysql://user:pass@hostname/dbname
                #     *   The table is the name of the table to use within the
                #     *   specified database.
                #     */
                #    global $addrbook_dsn, $addrbook_table;
                echo "\$addrbook_dsn     = '';"

                #    /**
                #     * This is the name of the table you want to store the address book
                #     * data in, it defaults to 'address'
                #     */
                echo "\$addrbook_table   = 'address';"
                echo

                #    /**
                #     * If you want to store your users preferences in a database then
                #     * you need to set this DSN to a valid value. The format for this is:
                #     * mysql://user:pass@hostname/dbname
                #     * Where mysql can be one of the databases PHP supports, the most common
                #     * of these are mysql, msql and pgsql
                #     * If the DSN is left empty (hit space and then return) the database
                #     * related code for address books will not be used
                #     */
                #    global $prefs_dsn, $prefs_table;
                echo "\$prefs_dsn        = '';"

                #    /**
                #     * This is the name of the table you want to store the preferences
                #     * data in, it defaults to 'userprefs'
                #     */
                echo "\$prefs_table      = 'userprefs';"

                #    /**
                #     * This is the name of the field in which you want to store the
                #     * username of the person the prefs are for. It default to 'user'
                #     * which clashes with a reserved keyword in PostgreSQL so this
                #     * will need to be changed for that database at least
                #     */
                echo "\$prefs_user_field = 'user';"

                #    /**
                #     * This is the name of the field in which you want to store the
                #     * preferences keyword. It defaults to 'prefkey'
                #     */
                echo "\$prefs_key_field  = 'prefkey';"

                #    /**
                #     * This is the name of the field in which you want to store the
                #     * preferences value. It defaults to 'prefval'
                #     */
                echo "\$prefs_val_field  = 'prefval';"

                #    /**
                #     * If you want to store your global address book in a database then
                #     * you need to set this DSN to a valid value. The format for this is:
                #     * mysql://user:pass\@hostname/dbname
                #     * Where mysql can be one of the databases PHP supports, the most common
                #     * of these are mysql, msql and pgsql
                #     * If the DSN is left empty (hit space and then return) the database
                #     * related code for global SQL address book will not be used
                #     */
                echo "\$addrbook_global_dsn       = '';"               # (SM v1.4.5)
                echo "\$addrbook_global_table     = 'global_abook';"   # (SM v1.4.5)
                echo "\$addrbook_global_writeable = false;"            # (SM v1.4.5)
                echo "\$addrbook_global_listing   = false;"            # (SM v1.4.5)

                #    /**
                #     * LDAP server(s)
                #     *   Array of arrays with LDAP server parameters. See
                #     *   functions/abook_ldap_server.php for a list of possible
                #     *   parameters
                #     *
                #     * EXAMPLE:
                #     *   $ldap_server[0] = Array(
                #     *       'host' => 'memberdir.netscape.com',
                #     *       'name' => 'Netcenter Member Directory',
                #     *       'base' => 'ou=member_directory,o=netcenter.com'
                #     *   );
                #     */
                #
                #    global $ldap_server;
                # $ldap_server[0] = array(
                #     'host' => '<ip oder fqdn>',
                #     'base' => 'ou=contacts,ou=addressbook,dc=ps,dc=lan',
                #     'name' => 'LDAP: eis.ps.lan',
                #     'port' => 389,
                #     'charset' => 'utf-8',
                #     'maxrows' => 500,
                #     'binddn' => 'cn=ldapadmin,dc=ps,dc=lan',
                #     'bindpw' => '<password>',
                #     'protocol' => 3

                eval globldap_n='$WEBMAIL_'${wm_nbr}'_GLOBADDR_LDAP_N'

                if [ ${globldap_n} -gt 0 ]
                then
                    if ! check_active_php_ldap -quiet
                    then
                        # php-ldap support hasn't been activated
                        globldap_n=0
                        write_to_config_log -warn "Skipping WEBMAIL_${wm_nbr}_GLOBADDR_LDAP_.. because parameter"
                        write_to_config_log -warn -ff "PHP5_EXT_LDAP='yes' hasn't been set in apache2_php5 package."
                    fi

                    echo 

                    idx=1
                    while [ $idx -le $globldap_n ]
                    do
                        eval wm_globldap_active='$WEBMAIL_'${wm_nbr}'_GLOBADDR_LDAP_'${idx}'_ACTIVE'

                        if [ "${wm_globldap_active}" = "yes" ]
                        then
                            eval wm_globldap_info='$WEBMAIL_'${wm_nbr}'_GLOBADDR_LDAP_'${idx}'_INFO'
                            eval wm_globldap_hostport='$WEBMAIL_'${wm_nbr}'_GLOBADDR_LDAP_'${idx}'_HOST'
                            eval wm_globldap_basedn='$WEBMAIL_'${wm_nbr}'_GLOBADDR_LDAP_'${idx}'_BASEDN'
                            eval wm_globldap_auth='$WEBMAIL_'${wm_nbr}'_GLOBADDR_LDAP_'${idx}'_AUTH'
                            eval wm_globldap_binddn='$WEBMAIL_'${wm_nbr}'_GLOBADDR_LDAP_'${idx}'_BINDDN'
                            eval wm_globldap_bindpass='$WEBMAIL_'${wm_nbr}'_GLOBADDR_LDAP_'${idx}'_BINDPASS'
                            eval wm_globldap_charset='$WEBMAIL_'${wm_nbr}'_GLOBADDR_LDAP_'${idx}'_CHARSET'   # optional
                            eval wm_globldap_maxrows='$WEBMAIL_'${wm_nbr}'_GLOBADDR_LDAP_'${idx}'_MAXROWS'   # optional

                            # temporarily remove ldap(s):// protocol information
                            tmp_globldap_hostport=`echo "${wm_globldap_hostport}" | sed 's#^ldap.*://##'`

                            # check if port number has been given
                            echo "${tmp_globldap_hostport}" | grep -q ":"

                            if [ $? -eq 0 ]
                            then
                                # split hostname and port
                                wm_globldap_port=`echo "${tmp_globldap_hostport}" | cut -d: -f2`
                                wm_globldap_host=`echo "${wm_globldap_hostport}" | sed "s#:${wm_globldap_port}##"`
                            else
                                # hostname only
                                wm_globldap_host="${wm_globldap_hostport}"

                                # set default ldap port
                                echo "${wm_globldap_hostport}" | grep -q "^ldaps:"

                                if [ $? -eq 0 ]
                                then
                                    # ldaps
                                    wm_globldap_port='636'
                                else
                                    # ldap
                                    wm_globldap_port='389'
                                fi
                            fi

                            echo "\$ldap_server[`expr ${idx} - 1`] = array("
                            echo "    'host'     => '${wm_globldap_host}',"
                            echo "    'name'     => '${wm_globldap_info}',"
                            echo "    'base'     => '${wm_globldap_basedn}',"

                            if [ "${wm_globldap_auth}" = "yes" ]
                            then
                                echo "    'binddn'   => '${wm_globldap_binddn}',"
                                echo "    'bindpw'   => '${wm_globldap_bindpass}',"
                            fi

                            echo "    'port'     => ${wm_globldap_port},"

                            if [ "${wm_globldap_charset}" != "" ]
                            then
                                echo "    'charset'  => '${wm_globldap_charset}',"   # optional
                            else
                                # default value
                                echo "    'charset'  => '${wm_general_def_charset}',"
                            fi

                            if [ "${wm_globldap_maxrows}" != "" ]
                            then
                                echo "    'maxrows'  => ${wm_globldap_maxrows},"     # optional
                            else
                                # default value
                                echo "    'maxrows'  => 150,"
                            fi

                            echo "    'protocol' => 3"
                            echo ");"
                        else
                            # deactivated
                            write_to_config_log -info "Skipping WEBMAIL_${wm_nbr}_GLOBADDR_LDAP_${idx}_.. because"
                            write_to_config_log -info -ff "it has been deactivated in the configuration file."
                        fi

                        idx=`expr ${idx} + 1`
                    done
                fi

                echo

                #    /**
                #     * PHP session name
                #     *   Leave this alone unless you know what you are doing.
                #     */
                #    global $session_name;
                echo "\$session_name = 'SQMSESSID';"

                #    /**
                #     * disable_security_tokens
                #     *   This option allows you to turn off the security checks in the forms
                #     *   that SquirrelMail generates. It is NOT RECOMMENDED that you disable
                #     *   this feature - otherwise, your users may be exposed to phishing and
                #     *   other attacks.
                #     *   Unless you know what you are doing, you should leave this set to 'false'.
                #     */
                echo "\$disable_security_tokens = false;"

                #    /**
                #     *    This option allows you to enable referal checks for all page requests
                #     *    made to SquirrelMail.  This can help ensure that page requests came
                #     *    from the same server and not from an attacker's site (usually the
                #     *    result of a XSS or phishing attack).  To enable referal checking,
                #     *    this setting can be set to the domain where your SquirrelMail is
                #     *    being hosted (usually the same as the Domain setting under Server
                #     *    Settings).  For example, it could be \"example.com\", or if you
                #     *    use a plugin (such as Login Manager) to host SquirrelMail on more
                #     *    than one domain, you can set this to \"###DOMAIN###\" to tell it
                #     *    to use the current domain.
                #     */

                if [ "${wm_server_domain_check}" = "yes" ]
                then
                    echo "\$check_referrer          = '###DOMAIN###';"
                fi

                echo

                echo "@include SM_PATH . 'config/config_local.php';"
                echo

                echo "/**"
                echo " * Make sure there are no characters after the PHP closing"
                echo " * tag below (including newline characters and whitespace)."
                echo " * Otherwise, that character will cause the headers to be"
                echo " * sent and regular output to begin, which will majorly screw"
                echo " * things up when we try to send more headers later."
                echo " */"
                echo "?>"
            } > $wm_doc_root/config/config.php
        fi
    fi
}

#----------------------------------------------------------------------------------------
# generate squirrelmail stop configuration
# $1 - webmail instance
#----------------------------------------------------------------------------------------
create_squirrelmail_stop_conf ()
{
    wm_nbr=$1
    eval wm_doc_root='$WEBMAIL_'${wm_nbr}'_DOCUMENT_ROOT'
    eval wm_active='$WEBMAIL_'${wm_nbr}'_ACTIVE'
    eval wm_orga_title='$WEBMAIL_'${wm_nbr}'_ORGA_TITLE'
    eval wm_orga_signout_page='$WEBMAIL_'${wm_nbr}'_ORGA_SIGNOUT_PAGE'
    eval wm_orga_top_frame='$WEBMAIL_'${wm_nbr}'_ORGA_TOP_FRAME'

    if [ "$START_WEBMAIL" != "yes" -o "$wm_active" != "yes" ]
    then
        mecho "creating webmail stop configuration ..."

        if [ -d $wm_doc_root/config ]
        then
            {
                echo "<?php"
                echo
                echo "/********************************************************************************"
                echo " * webmail stop configuration file generated by $pgmname"
                echo " *"
                echo " * Do not edit this file, edit $webmailfile"
                echo " * Creation date: `date`"
                echo " *******************************************************************************/"
                echo

                echo "\$config_version   = '1.4.0';"
                echo "\$config_use_color = 2;"
                echo

                echo "/* organisation settings */"
                echo "\$org_name        = '+++ Webmail has been disabled! +++';"
                echo "\$org_logo        = '../images/eisfair_stop_sign.png';"
                echo "\$org_logo_width  = 156;"
                echo "\$org_logo_height = 156;"
                echo "\$org_title       = \"$wm_orga_title\";"
                echo "\$signout_page    = '$wm_orga_signout_page';"
                echo "\$frame_top       = '$wm_orga_top_frame';"
                echo
                echo "/**"
                echo " * Make sure there are no characters after the PHP closing"
                echo " * tag below (including newline characters and whitespace)."
                echo " * Otherwise, that character will cause the headers to be"
                echo " * sent and regular output to begin, which will majorly screw"
                echo " * things up when we try to send more headers later."
                echo " */"
                echo "?>"
            } > $wm_doc_root/config/config.php
        fi
    fi
}

#----------------------------------------------------------------------------------------
#  create global address book
# $1 - webmail instance
#----------------------------------------------------------------------------------------
create_global_addressbook ()
{
    wm_nbr=$1
    if [ $wm_nbr -ne 0 -a $wm_nbr -le $WEBMAIL_N ]
    then
        # create directory if it doesn't exist
        if [ ! -f $webmail_data_path ]
        then
            mkdir -p $webmail_data_path
        fi

        # nickname|firstname|lastname|email|info
        eval globaddr_n='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_N'

        if [ ${globaddr_n} -gt 0 ]
        then
            # create global address book
            mecho "- creating global addressbook ..."

            # create new file
            > ${generate_globaddr}-${wm_nbr}

            idx=1
            while [ $idx -le $globaddr_n ]
            do
                eval glob_nick='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_'${idx}'_NICKNAME'
                eval glob_fname='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_'${idx}'_FIRSTNAME'
                eval glob_lname='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_'${idx}'_LASTNAME'
                eval glob_email='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_'${idx}'_EMAIL'
                eval glob_info='$WEBMAIL_'${wm_nbr}'_GLOBADDR_FILE_'${idx}'_INFO'

                echo "$glob_nick|$glob_fname|$glob_lname|$glob_email|$glob_info" >> ${generate_globaddr}-${wm_nbr}

                idx=`expr $idx + 1`
            done
        else
            # delete global address book and create empty file
            > ${generate_globaddr}-${wm_nbr}
        fi
    fi
}

#----------------------------------------------------------------------------------------
# check for administrator plugin
# $1 - webmail instance
#----------------------------------------------------------------------------------------
check_for_administrator_plugin ()
{
    wm_nbr=$1
    eval wm_doc_root='$WEBMAIL_'${wm_nbr}'_DOCUMENT_ROOT'
    eval wm_plugin_admin='$WEBMAIL_'${wm_nbr}'_PLUGINS_ADMINUSER'
    
    if [ -f $wm_doc_root/config/config.php ]
    then
        mecho "- checking administrator plugin ..."

        grep "^\$plugin" $wm_doc_root/config/config.php|cut -d= -f2|grep "administrator" > /dev/null

        if [ $? -eq 0 ]
        then
            # administrator-plugin has been enabled
            if [ "$wm_plugin_admin" = "" ]
            then
                write_to_config_log -error "You have enabled the \"administrator\" plugin but haven't set the variable"
                write_to_config_log -error -ff "\$WEBMAIL_${wm_nbr}_PLUGINS_ADMINUSER. Use this plugin at your own risk!"
            else
                write_to_config_log -warn "You have enabled the \"administrator\" plugin for the user \"$wm_plugin_admin\"."
                write_to_config_log -warn -ff "Use this plugin at your own risk!"
            fi
        fi
    fi
}

#----------------------------------------------------------------------------------------
# set access rights
#----------------------------------------------------------------------------------------
set_webmail_access_rights ()
{
    mecho "checking directories ..."

    # check directories
    for DNAME in $webmail_data_path $webmail_attach_path $webmail_log_path
    do
        if [ ! -f $DNAME ]
        then
            mkdir -p $DNAME
        fi
    done

    mecho "setting access rights ..."

    idx=1
    while [ $idx -le $WEBMAIL_N ]
    do
        # webmail root directory
        eval wm_doc_root='$WEBMAIL_'${idx}'_DOCUMENT_ROOT'
        eval wm_plugin_admin='$WEBMAIL_'${idx}'_PLUGINS_ADMINUSER'
        eval wm_globaddr_write='$WEBMAIL_'${idx}'_GLOBADDR_FILE_WRITEABLE'

        if [ -d $wm_doc_root ]
        then
            chown -R $webmail_apache_user  $wm_doc_root
            chgrp -R $webmail_apache_group $wm_doc_root

            chmod -R 444 $wm_doc_root

            find $wm_doc_root -type d -exec chmod 755 {} \;
        fi

        if [ -d $wm_doc_root/themes ]
        then
            # themes directory
            chown -f -R $webmail_apache_user  $wm_doc_root/themes
            chgrp -f -R $webmail_apache_group $wm_doc_root/themes
        fi

        if [ -d $wm_doc_root/plugins ]
        then
            # plugins directory
            chown -f -R $webmail_apache_user  $wm_doc_root/plugins
            chgrp -f -R $webmail_apache_group $wm_doc_root/plugins
        fi

        # configuration file
        if [ -f $wm_doc_root/config/config.php ]
        then
            if [ "$wm_plugin_admin" = "" ]
            then
                chown -f -R $webmail_apache_user $wm_doc_root/config/config.php
                chmod 440 $wm_doc_root/config/config.php
            else
                # use admin user if admintrator plugin has been enabled
                eval wm_plugin_admin='$WEBMAIL_'${idx}'_PLUGINS_ADMINUSER'
                chown -f -R $wm_plugin_admin $wm_doc_root/config/config.php
                chmod 660 $wm_doc_root/config/config.php
            fi
            chgrp -f -R $webmail_apache_group $wm_doc_root/config/config.php
        fi

        # global addressbook
        if [ -f $generate_globaddr-${idx} ]
        then
            if [ "$wm_globaddr_write" = "yes" ]
            then
                chmod -f 640 $generate_globaddr-${idx}
            else
                chmod -f 440 $generate_globaddr-${idx}
            fi
        fi

        idx=`expr $idx + 1`
    done

    # data and attach directory
    if [ -d ${webmail_path} ]
    then
        chown -f -R $webmail_apache_user  ${webmail_path}
        chgrp -f -R $webmail_apache_group ${webmail_path}

        chmod -f 755 ${webmail_path}
    fi

    if [ -d $webmail_data_path ]
    then
        chmod -f 750 $webmail_data_path
    fi
    if [ -d $webmail_attach_path ]
    then
        chmod -f 730 $webmail_attach_path
    fi
    if [ -d $webmail_log_path ]
    then
        chmod -f 750 $webmail_log_path
    fi
}

#----------------------------------------------------------------------------------------
# create logrotate file
#----------------------------------------------------------------------------------------
create_logrotate ()
{
    mecho "creating logrotate configuration ..."

    # create directory if doesn't exist
    if [ ! -f $webmail_log_path ]
    then
        mkdir -p $webmail_log_path
    fi

    {
        #----------------------------------------------------------------------------------------
        print_short_header "$generate_logrotate" "$pgmname" "$webmail_version"
        #----------------------------------------------------------------------------------------

        if [ "$START_WEBMAIL" = "yes" ]
        then
            echo "$webmail_log_path/access_log {"
            echo "    rotate $WEBMAIL_LOG_COUNT"
            echo "    $WEBMAIL_LOG_INTERVAL"
            echo "    compress"
            echo "    missingok"
            echo "    notifempty"
            echo "    }"
            echo
        else
            rm -f $generate_logrotate > /dev/null
        fi
    } > $generate_logrotate
}

#----------------------------------------------------------------------------------------
# check IMAP server
#----------------------------------------------------------------------------------------
check_imap_server ()
{
    case ${MAIL_INSTALLED} in
        mail)
            # mail package
            if [ "${START_IMAP}" = "yes" ]
            then
                mecho "imap server is active - ok."
            else
                mecho -warn "imap server is inactive."
                write_to_config_log -warn "Parameter START_IMAP='yes' has not been set although a local mail"
                write_to_config_log -warn -ff "package has been installed! Email can't be retrieved!"
            fi
            ;;
        vmail)
            # vmail package
            if [ "${START_POP3IMAP}" = "yes" ]
            then
                mecho "imap server is active - ok."
            else
                mecho -warn "imap server is inactive!"
                write_to_config_log -warn "Parameter START_POP3IMAP='yes' has not been set although a local vmail"
                write_to_config_log -warn -ff "package has been installed! Email can't be retrieved!"
            fi
            ;;
        *)
            # none local
            mecho -warn "no local mail/vmail package found - imap server status cannot be evaluate!"
            write_to_config_log -warn "no local mail/vmail package found - imap server status cannot be evaluate!"
            ;;
    esac
}

#----------------------------------------------------------------------------------------
# add cron job
#----------------------------------------------------------------------------------------
add_cron_job ()
{
    mecho "creating cron job ..."

    # check for cron directory
    if [ ! -d $crontab_path ]
    then
        mkdir -p $crontab_path
    fi

    # write file
    {
        echo "#--------------------------------------------------------------------"
        echo "#  cron webmail file generated by ${pgmname} version: ${webmail_version}"
        echo "#"
        echo "#  Do not edit this file, edit ${webmailfile}"
        echo "#  Creation Date: ${EISDATE} Time: ${EISTIME}"
        echo "#--------------------------------------------------------------------"
        echo "$WEBMAIL_ATTACH_CRON_SCHEDULE ${webmail_path}/bin/remove_forgotten_attachments"
        echo
    } > $crontab_file

    # update crontab file
    /var/install/config.d/cron > /dev/null 2> /dev/null
}

#----------------------------------------------------------------------------------------
# delete cron job
#----------------------------------------------------------------------------------------
delete_cron_job ()
{
    mecho "deleting cron job ..."

    # check for crontab file
    if [ -f $crontab_file ]
    then
        # delete existing file
        rm -f $crontab_file

        # update crontab file
        /var/install/config.d/cron > /dev/null 2> /dev/null
    fi
}

#----------------------------------------------------------------------------------------
# check if unique document root
# $1 - webmail instance
#----------------------------------------------------------------------------------------
is_unique_docroot ()
{
    wm_nbr=$1
    wmret=0

    if [ $wm_nbr -gt 1 -a $WEBMAIL_N -ne 0 ]
    then
        eval wm_active1='$WEBMAIL_'${wm_nbr}'_ACTIVE'
        eval wm_doc_root1='$WEBMAIL_'${wm_nbr}'_DOCUMENT_ROOT'

        if [ "${wm_active1}" = "yes" ]
        then
            # active entry - compare it...
            mecho "- checking if document root is unique ..."

            idx=1
            while [ $idx -le $WEBMAIL_N ]
            do
                if [ $idx -ne $wm_nbr ]
                then
                    eval wm_active2='$WEBMAIL_'${idx}'_ACTIVE'

                    if [ "${wm_active2}" = "yes" ]
                    then
                        # active entry - compare it...
                        eval wm_doc_root2='$WEBMAIL_'${idx}'_DOCUMENT_ROOT'

                        if [ "$wm_doc_root1" = "$wm_doc_root2" ]
                        then
                            mecho "! Error: duplicate document roots (${wm_nbr} = ${idx}) found, please check configuration!"
                            write_to_config_log -error "Duplicate document roots (${wm_nbr} = ${idx}) found!"
                            wmret=1
                            break
                        fi
                    fi
                fi

                idx=`expr $idx + 1`
            done
        fi
    fi

    return ${wmret}
}

#----------------------------------------------------------------------------------------
# copy program files
# $1 - webmail instance
#----------------------------------------------------------------------------------------
copy_program_files ()
{
    wm_nbr=$1
    eval wm_doc_root='$WEBMAIL_'${wm_nbr}'_DOCUMENT_ROOT'

    mecho "- copying program files ($wm_doc_root) ..."
        
    if [ ! -f "$wm_doc_root/config/config.php" ]
    then
        # files don't exist - copy ...
        if [ ! -d "$wm_doc_root" ]
        then
            mkdir -p "$wm_doc_root"
        fi

        # extract squirrelmail software
        tar xzf ${webmail_path}/.install/sm.tgz -C "$wm_doc_root"

        # extract squirrelmail german language module
        tar xzf ${webmail_path}/.install/sm_de.tgz   -C "$wm_doc_root"

        # extract squirrelmail software modifications
        tar xzf ${webmail_path}/.install/sm_mod.tgz  -C "$wm_doc_root"

        # add document root to list
        echo "${wm_doc_root}:" >> "${docroot_addlist}"
    fi

    # create included locale files
    echo "# part of webmail package" > "${webmail_path}/de_DE-filelist-${wm_nbr}.txt"
    echo "# part of webmail package" > "${webmail_path}/en_US-filelist-${wm_nbr}.txt"
}

#----------------------------------------------------------------------------
# remove installed locale
# $1 - webmail instance
#----------------------------------------------------------------------------
remove_locale ()
{
    wm_nbr=$1

    locale_mask="??_??-filelist-${wm_nbr}.txt"

    # get document root
    eval wm_root_path=${testroot}'$WEBMAIL_'${wm_nbr}'_DOCUMENT_ROOT'

    if [ -d "${wm_doc_root}" ]
    then
        for FLIST in ${webmail_path}/${locale_mask}
        do
            # example: de_DE-filelist-1.txt
            disp_code=`basename "${FLIST}"|cut -c1-5`
            ccode=`echo "${disp_code}" | tr 'A-Z' 'a-z'`

            # check if included locale should be removed
            if [ "$ccode" != "de_de" -a "$ccode" != "en_us" ]
            then
                # locale not included - go on ...
                mecho "- removing locale files (${disp_code}) ..."

                # remove files
                for FDNAME in `cat "${FLIST}"`
                do
                    # check for comment
                    echo "${wm_root_path}/${FDNAME}" | grep -q "^#"

                    if [ $? -ne 0 ]
                    then
                        # no comment - go on...
                        if [ -f "${wm_root_path}/${FDNAME}" ]
                        then
                            # remove file
                            rm -f "${wm_root_path}/${FDNAME}"
                        fi
                    fi
                done
            fi

            # remove directories
            if [ -d "${wm_root_path}/locale/${disp_code}/LC_MESSAGES" ]
            then
                rmdir --ignore-fail-on-non-empty "${wm_root_path}/locale/${disp_code}/LC_MESSAGES"
            fi
            if [ -d "${wm_root_path}/locale/${disp_code}" ]
            then
                rmdir --ignore-fail-on-non-empty "${wm_root_path}/locale/${disp_code}"
            fi
            if [ -d "${wm_root_path}/help/${disp_code}" ]
            then
                rmdir --ignore-fail-on-non-empty "${wm_root_path}/help/${disp_code}"
            fi

            # remove list file
            if [ -f "${FLIST}" ]
            then
                rm -f "${FLIST}"
            fi
        done

        # remove document root directory
        rmdir --ignore-fail-on-non-empty "${wm_doc_root}"
    fi
}

#----------------------------------------------------------------------------------------
# remove installed program files
# $1 - webmail instance
#----------------------------------------------------------------------------------------
remove_program_files ()
{
    wm_nbr="$1"

    eval wm_doc_root='$WEBMAIL_'${wm_nbr}'_DOCUMENT_ROOT'

    if [ -d "${wm_doc_root}" ]
    then
        mecho "- removing program files (${wm_doc_root}) ..."

        if [ -f "${docroot_filelist}" ]
        then
            while read line
            do
                # check for comment
                echo "${line}" | grep -q "^#"

                if [ $? -ne 0 ]
                then
                    # no comment - go on...
                    FDNAME="${testroot}${wm_doc_root}/${line}"

                    if [ -d "${FDNAME}" ]
                    then
                        # remove directory
                        rmdir --ignore-fail-on-non-empty "${FDNAME}"
                    elif [ -f "${FDNAME}" ]
                    then
                        # remove file
                        rm -f "${FDNAME}"
                    fi
                fi
            done < "${docroot_filelist}"
        else
            mecho -error "missing ${docroot_filelist} file, error removing package files!"
        fi

        # remove document root directory
        rmdir --ignore-fail-on-non-empty "${wm_doc_root}"
    fi
}

#----------------------------------------------------------------------------------------
# purge document roots
#----------------------------------------------------------------------------------------
purge_document_roots ()
{
    mecho "purging document root directories ..."

    # backup doccument root deletion list
    /var/install/bin/backup-file -quiet "${docroot_dellist}" backup

    if [ -f "${docroot_addlist}" ]
    then
        # document root list exists
        /var/install/bin/backup-file -quiet "${docroot_addlist}" backup

        if [ -f "${docroot_dellist}" ]
        then
            # append current document root list to deletion list
            cat "${docroot_dellist}" "${docroot_addlist}" | sort > "${docroot_tmplist}"
            mv  "${docroot_tmplist}" "${docroot_dellist}"
        else
            # copy current document root list to deletion list
            cp  "${docroot_addlist}" "${docroot_dellist}"
        fi

        # remove previous document root list
        rm -f "${docroot_addlist}"
    fi

    if [ $WEBMAIL_N -ne 0 ]
    then
        idx=1
        while [ $idx -le $WEBMAIL_N ]
        do
            eval wm_doc_root='$WEBMAIL_'${idx}'_DOCUMENT_ROOT'

            # add entry to document root list
            echo "${wm_doc_root}:" >> "${docroot_addlist}"

            if [ -f "${docroot_dellist}" ]
            then
                grep -q "^${wm_doc_root}:" "${docroot_dellist}"

                if [ $? -eq 0 ]
                then
                    # remove document root from deletion list
                    grep -v "^${wm_doc_root}:" "${docroot_dellist}" > "${docroot_tmplist}"
                    mv  "${docroot_tmplist}" "${docroot_dellist}"
                fi
            fi

            idx=`expr $idx + 1`
        done
    fi

    if [ -f "${docroot_dellist}" ]
    then
        if [ -f "${docroot_tmplist}" ]
        then
            rm -f "${docroot_tmplist}"
        fi

        cut -d: -f1 "${docroot_dellist}" |\
        while read line
        do
            echo "${line}" | grep -q "^#"

            if [ $? -ne 0 ]
            then
                if [ -d "${line}" ]
                then
                    # directory exists
                    /var/install/bin/ask "Directory '${line}' is not used anymore, delete it" 'n' > /tmp/ask.$$ <$tty
                    rc=$?
                    yesno=`cat /tmp/ask.$$|tr 'A-Z' 'a-z'`
                    rm -f /tmp/ask.$$
                    if [ $rc = 255 ]
                    then
                        rm $tmpfile
                        exit 1
                    fi

                    if [ "${yesno}" = "yes" ]
                    then
                        # delete webmail program directory
                        remove_locale ${idx}
                        remove_program_files ${idx}
                    else
                        # preserve directory
                        echo "${line}:" >> "${docroot_tmplist}"
                    fi
                fi
            fi
        done

        if [ -f "${docroot_tmplist}" -a -s "${docroot_tmplist}" ]
        then
            # tmp file exists and contains data, replace deletion list
            mv "${docroot_tmplist}" "${docroot_dellist}"
        else
            # remove deletion list
            if [ -f "${docroot_dellist}" ]
            then
                rm -f "${docroot_dellist}"
            fi
        fi
    fi

    # delete temp file
    if [ -f "${docroot_tmplist}" ]
    then
        rm -f "${docroot_tmplist}"
    fi
}

#========================================================================================
# Main
#========================================================================================

case "$1"
in
    removecron)
        delete_cron_job
        ;;

    *)
        if [ "${START_WEBMAIL}" = "yes" ]
        then
            # generate all configuration files
            mecho "version: ${webmail_version}"

            if check_installed_mail
            then
                # mail
                MAIL_INSTALLED='mail'

                if [ -f ${mailfile} ]
                then
                    # mail package found
                    . ${mailfile}
                fi
            elif check_installed_vmail
            then
                # vmail
                MAIL_INSTALLED='vmail'

                if [ -f ${vmailfile} ]
                then
                    # vmail package found
                    . ${vmailfile}
                fi
            else
                # no local mail or vmail package installed
                MAIL_INSTALLED='none'
            fi

            # check for vmail plugin
            check_installed_vmail_plugin -quiet

            VMAIL_PLUGIN_INSTALLED=$?

            write_to_config_log -header

            check_imap_server

            wmidx=1
            while [ $wmidx -le $WEBMAIL_N ]
            do
                mecho "processing webmail instance ($wmidx) ..."

                eval active='$WEBMAIL_'${wmidx}'_ACTIVE'

                if is_unique_docroot $wmidx
                then
                    # document root is unique
                    if [ "$active" = "yes" ]
                    then
                        copy_program_files $wmidx

                        if [ ${VMAIL_PLUGIN_INSTALLED} -eq 0 ]
                        then
                            # configure vmail plugin
                            mecho "- copying/configuring vmail plugin ..."
                            ${webmail_vmail_config_file} ${wmidx}
                        fi

                        create_squirrelmail_conf $wmidx
                        create_global_addressbook $wmidx
                        check_for_administrator_plugin $wmidx
                    else
                        mecho -warn "- instance has been disabled."
                        write_to_config_log -warn "webmail instance ($wmidx) has been disabled."

                        create_squirrelmail_stop_conf $wmidx
                    fi
                fi

                wmidx=`expr $wmidx + 1`
            done

            create_logrotate
            add_cron_job

            set_webmail_access_rights

            purge_document_roots

            mecho "finished."

            compress_config_log
            display_config_log
        else
            # generate stop configuration file
            mecho "version: $webmail_version"

            idx=1
            while [ $idx -le $WEBMAIL_N ]
            do
                create_squirrelmail_stop_conf $idx

                idx=`expr $idx + 1`
            done

            rm -f $generate_logrotate
            delete_cron_job
        fi

        # restart web server
        if /var/install/bin/ask "Do you want to restart the webserver now (recommended)" "yes"
        then
            /etc/init.d/apache2 restart
        fi
        ;;
esac

#========================================================================================
# End
#========================================================================================
exit 0
